@@ -97,14 +97,17 @@ void sal_disable_logs() {
 
 void sal_add_pending_auth(Sal *sal, SalOp *op){
 	if (ms_list_find(sal->pending_auths,op)==NULL){
-		sal->pending_auths=ms_list_append(sal->pending_auths,sal_op_ref(op));
+		sal->pending_auths=ms_list_append(sal->pending_auths,op);
+		op->has_auth_pending=TRUE;
 	}
 }
 
 void sal_remove_pending_auth(Sal *sal, SalOp *op){
-	if (ms_list_find(sal->pending_auths,op)){
-		sal->pending_auths=ms_list_remove(sal->pending_auths,op);
-		sal_op_unref(op);
+	if (op->has_auth_pending){
+		op->has_auth_pending=FALSE;
+		if (ms_list_find(sal->pending_auths,op)){
+			sal->pending_auths=ms_list_remove(sal->pending_auths,op);
+		}
 	}
 }
 
@@ -115,7 +118,13 @@ void sal_process_authentication(SalOp *op) {
 	belle_sip_list_t* auth_list=NULL;
 	belle_sip_auth_event_t* auth_event;
 	belle_sip_response_t *response=belle_sip_transaction_get_response((belle_sip_transaction_t*)op->pending_auth_transaction);
-	
+	belle_sip_header_from_t *from=belle_sip_message_get_header_by_type(initial_request,belle_sip_header_from_t);
+	belle_sip_uri_t *from_uri=belle_sip_header_address_get_uri((belle_sip_header_address_t*)from);
+
+	if (strcasecmp(belle_sip_uri_get_host(from_uri),"anonymous.invalid")==0){
+		/*prefer using the from from the SalOp*/
+		from_uri=belle_sip_header_address_get_uri((belle_sip_header_address_t*)sal_op_get_from_address(op));
+	}
 
 	if (op->dialog && belle_sip_dialog_get_state(op->dialog)==BELLE_SIP_DIALOG_CONFIRMED) {
 		new_request = belle_sip_dialog_create_request_from(op->dialog,initial_request);
@@ -131,8 +140,8 @@ void sal_process_authentication(SalOp *op) {
 		ms_error("sal_process_authentication() op=[%p] cannot obtain new request from dialog.",op);
 		return;
 	}
-	
-	if (belle_sip_provider_add_authorization(op->base.root->prov,new_request,response,&auth_list)) {
+
+	if (belle_sip_provider_add_authorization(op->base.root->prov,new_request,response,from_uri,&auth_list,op->base.realm)) {
 		if (is_within_dialog) {
 			sal_op_send_request(op,new_request);
 		} else {
@@ -151,7 +160,10 @@ void sal_process_authentication(SalOp *op) {
 		}
 	}
 	/*always store auth info, for case of wrong credential*/
-	if (op->auth_info) sal_auth_info_delete(op->auth_info);
+	if (op->auth_info) {
+		sal_auth_info_delete(op->auth_info);
+		op->auth_info=NULL;
+	}
 	if (auth_list){
 		auth_event=(belle_sip_auth_event_t*)(auth_list->data);
 		op->auth_info=sal_auth_info_create(auth_event);
@@ -162,8 +174,8 @@ void sal_process_authentication(SalOp *op) {
 static void process_dialog_terminated(void *sal, const belle_sip_dialog_terminated_event_t *event){
 	belle_sip_dialog_t* dialog =  belle_sip_dialog_terminated_event_get_dialog(event);
 	SalOp* op = belle_sip_dialog_get_application_data(dialog);
-	if (op && op->callbacks.process_dialog_terminated) {
-		op->callbacks.process_dialog_terminated(op,event);
+	if (op && op->callbacks && op->callbacks->process_dialog_terminated) {
+		op->callbacks->process_dialog_terminated(op,event);
 	} else {
 		ms_error("sal process_dialog_terminated no op found for this dialog [%p], ignoring",dialog);
 	}
@@ -177,8 +189,8 @@ static void process_io_error(void *user_ctx, const belle_sip_io_error_event_t *e
 		op = (SalOp*)belle_sip_transaction_get_application_data(BELLE_SIP_TRANSACTION(client_transaction));
 		/*also reset auth count on IO error*/
 		op->auth_requests=0;
-		if (op->callbacks.process_io_error) {
-				op->callbacks.process_io_error(op,event);
+		if (op->callbacks && op->callbacks->process_io_error) {
+			op->callbacks->process_io_error(op,event);
 		}
 	} else {
 		/*ms_error("sal process_io_error not implemented yet for non transaction");*/
@@ -192,7 +204,7 @@ static void process_request_event(void *ud, const belle_sip_request_event_t *eve
 	belle_sip_request_t* req = belle_sip_request_event_get_request(event);
 	belle_sip_dialog_t* dialog=belle_sip_request_event_get_dialog(event);
 	belle_sip_header_address_t* origin_address;
-	belle_sip_header_address_t* address;
+	belle_sip_header_address_t* address=NULL;
 	belle_sip_header_from_t* from_header;
 	belle_sip_header_to_t* to;
 	belle_sip_response_t* resp;
@@ -246,14 +258,22 @@ static void process_request_event(void *ud, const belle_sip_request_event_t *eve
 		return;
 	}else {
 		ms_error("sal process_request_event not implemented yet for method [%s]",belle_sip_request_get_method(req));
-		resp=belle_sip_response_create_from_request(req,501);
+		resp=belle_sip_response_create_from_request(req,405);
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(resp)
+									,BELLE_SIP_HEADER(belle_sip_header_allow_create("INVITE, CANCEL, ACK, BYE, SUBSCRIBE, NOTIFY, MESSAGE, OPTIONS, INFO")));
 		belle_sip_provider_send_response(sal->prov,resp);
 		return;
 	}
 
 	if (!op->base.from_address)  {
-		address=belle_sip_header_address_create(belle_sip_header_address_get_displayname(BELLE_SIP_HEADER_ADDRESS(from_header))
-														,belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(from_header)));
+		if (belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(from_header)))
+			address=belle_sip_header_address_create(belle_sip_header_address_get_displayname(BELLE_SIP_HEADER_ADDRESS(from_header))
+					,belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(from_header)));
+		else if ((belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(from_header))))
+			address=belle_sip_header_address_create2(belle_sip_header_address_get_displayname(BELLE_SIP_HEADER_ADDRESS(from_header))
+					,belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(from_header)));
+		else
+			ms_error("Cannot not find from uri from request [%p]",req);
 		sal_op_set_from_address(op,(SalAddress*)address);
 		belle_sip_object_unref(address);
 	}
@@ -264,8 +284,15 @@ static void process_request_event(void *ud, const belle_sip_request_event_t *eve
 
 	if (!op->base.to_address) {
 		to=belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(req),belle_sip_header_to_t);
-		address=belle_sip_header_address_create(belle_sip_header_address_get_displayname(BELLE_SIP_HEADER_ADDRESS(to))
-												,belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(to)));
+		if (belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(to)))
+			address=belle_sip_header_address_create(belle_sip_header_address_get_displayname(BELLE_SIP_HEADER_ADDRESS(to))
+					,belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(to)));
+		else if ((belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(to))))
+			address=belle_sip_header_address_create2(belle_sip_header_address_get_displayname(BELLE_SIP_HEADER_ADDRESS(to))
+					,belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(to)));
+		else
+			ms_error("Cannot not find to uri from request [%p]",req);
+
 		sal_op_set_to_address(op,(SalAddress*)address);
 		belle_sip_object_unref(address);
 	}
@@ -288,8 +315,8 @@ static void process_request_event(void *ud, const belle_sip_request_event_t *eve
 	sal_op_set_privacy_from_message(op,(belle_sip_message_t*)req);
 
 	sal_op_assign_recv_headers(op,(belle_sip_message_t*)req);
-	if (op->callbacks.process_request_event) {
-		op->callbacks.process_request_event(op,event);
+	if (op->callbacks && op->callbacks->process_request_event) {
+		op->callbacks->process_request_event(op,event);
 	} else {
 		ms_error("sal process_request_event not implemented yet");
 	}
@@ -300,7 +327,7 @@ static void process_response_event(void *user_ctx, const belle_sip_response_even
 	belle_sip_client_transaction_t* client_transaction = belle_sip_response_event_get_client_transaction(event);
 	belle_sip_response_t* response = belle_sip_response_event_get_response(event);
 	int response_code = belle_sip_response_get_status_code(response);
-	
+
 	if (!client_transaction) {
 		ms_warning("Discarding stateless response [%i]",response_code);
 		return;
@@ -313,9 +340,8 @@ static void process_response_event(void *user_ctx, const belle_sip_response_even
 			belle_sip_message("Op is terminated, nothing to do with this [%i]",response_code);
 			return;
 		}
-		if (!op->base.remote_ua) {
-			sal_op_set_remote_ua(op,BELLE_SIP_MESSAGE(response));
-		}
+		/*do it all the time, since we can receive provisional responses from a different instance than the final one*/
+		sal_op_set_remote_ua(op,BELLE_SIP_MESSAGE(response));
 
 		if(remote_contact) {
 			__sal_op_set_remote_contact(op, belle_sip_header_get_unparsed_value(BELLE_SIP_HEADER(remote_contact)));
@@ -326,15 +352,14 @@ static void process_response_event(void *user_ctx, const belle_sip_response_even
 		}
 
 		sal_op_assign_recv_headers(op,(belle_sip_message_t*)response);
-		
-		if (op->callbacks.process_response_event) {
+
+		if (op->callbacks && op->callbacks->process_response_event) {
 			/*handle authorization*/
 			switch (response_code) {
-				case 200: 
+				case 200:
 					break;
 				case 401:
 				case 407:
-					/*belle_sip_transaction_set_application_data(BELLE_SIP_TRANSACTION(client_transaction),NULL);*//*remove op from trans*/
 					if (op->state == SalOpStateTerminating && strcmp("BYE",belle_sip_request_get_method(request))!=0) {
 						/*only bye are completed*/
 						belle_sip_message("Op is in state terminating, nothing else to do ");
@@ -363,22 +388,23 @@ static void process_response_event(void *user_ctx, const belle_sip_response_even
 				/*not an auth request*/
 				op->auth_requests=0;
 			}
-			op->callbacks.process_response_event(op,event);
+			op->callbacks->process_response_event(op,event);
 		} else {
 			ms_error("Unhandled event response [%p]",event);
 		}
 	}
-
 }
+
 static void process_timeout(void *user_ctx, const belle_sip_timeout_event_t *event) {
 	belle_sip_client_transaction_t* client_transaction = belle_sip_timeout_event_get_client_transaction(event);
 	SalOp* op = (SalOp*)belle_sip_transaction_get_application_data(BELLE_SIP_TRANSACTION(client_transaction));
-	if (op && op->callbacks.process_timeout) {
-		op->callbacks.process_timeout(op,event);
+	if (op && op->callbacks && op->callbacks->process_timeout) {
+		op->callbacks->process_timeout(op,event);
 	} else {
 		ms_error("Unhandled event timeout [%p]",event);
 	}
 }
+
 static void process_transaction_terminated(void *user_ctx, const belle_sip_transaction_terminated_event_t *event) {
 	belle_sip_client_transaction_t* client_transaction = belle_sip_transaction_terminated_event_get_client_transaction(event);
 	belle_sip_server_transaction_t* server_transaction = belle_sip_transaction_terminated_event_get_server_transaction(event);
@@ -388,16 +414,15 @@ static void process_transaction_terminated(void *user_ctx, const belle_sip_trans
 	if(client_transaction)
 		trans=BELLE_SIP_TRANSACTION(client_transaction);
 	 else
-		 trans=BELLE_SIP_TRANSACTION(server_transaction);
+		trans=BELLE_SIP_TRANSACTION(server_transaction);
 
 	op = (SalOp*)belle_sip_transaction_get_application_data(trans);
-	if (op && op->callbacks.process_transaction_terminated) {
-		op->callbacks.process_transaction_terminated(op,event);
+	if (op && op->callbacks && op->callbacks->process_transaction_terminated) {
+		op->callbacks->process_transaction_terminated(op,event);
 	} else {
 		ms_message("Unhandled transaction terminated [%p]",trans);
 	}
-	if (op && client_transaction) sal_op_unref(op); /*because every client transaction ref op*/
-
+	if (op) sal_op_unref(op); /*because every transaction ref op*/
 }
 
 
@@ -415,15 +440,21 @@ static void process_auth_requested(void *sal, belle_sip_auth_event_t *event) {
 Sal * sal_init(){
 	belle_sip_listener_callbacks_t listener_callbacks;
 	Sal * sal=ms_new0(Sal,1);
+
+	/*belle_sip_object_enable_marshal_check(TRUE);*/
 	sal->auto_contacts=TRUE;
+
+	/*first create the stack, which initializes the belle-sip object's pool for this thread*/
+	belle_sip_set_log_handler(_belle_sip_log);
+	sal->stack = belle_sip_stack_new(NULL);
+
 	sal->user_agent=belle_sip_header_user_agent_new();
-#if defined(PACKAGE_NAME) && defined(LINPHONE_VERSION)
-	belle_sip_header_user_agent_add_product(sal->user_agent, PACKAGE_NAME "/" LINPHONE_VERSION);
+#if defined(PACKAGE_NAME) && defined(LIBLINPHONE_VERSION)
+	belle_sip_header_user_agent_add_product(sal->user_agent, PACKAGE_NAME "/" LIBLINPHONE_VERSION);
 #endif
 	sal_append_stack_string_to_user_agent(sal);
 	belle_sip_object_ref(sal->user_agent);
-	belle_sip_set_log_handler(_belle_sip_log);
-	sal->stack = belle_sip_stack_new(NULL);
+
 	sal->prov = belle_sip_stack_create_provider(sal->stack,NULL);
 	sal_nat_helper_enable(sal,TRUE);
 	memset(&listener_callbacks,0,sizeof(listener_callbacks));
@@ -439,8 +470,10 @@ Sal * sal_init(){
 	sal->tls_verify=TRUE;
 	sal->tls_verify_cn=TRUE;
 	sal->refresher_retry_after=60000; /*default value in ms*/
+	sal->enable_sip_update=TRUE;
 	return sal;
 }
+
 void sal_set_user_pointer(Sal *sal, void *user_data){
 	sal->up=user_data;
 }
@@ -514,6 +547,8 @@ void sal_uninit(Sal* sal){
 	belle_sip_object_unref(sal->prov);
 	belle_sip_object_unref(sal->stack);
 	belle_sip_object_unref(sal->listener);
+	if (sal->supported) belle_sip_object_unref(sal->supported);
+	ms_list_free_with_data(sal->supported_tags,ms_free);
 	if (sal->uuid) ms_free(sal->uuid);
 	if (sal->root_ca) ms_free(sal->root_ca);
 	ms_free(sal);
@@ -532,54 +567,90 @@ int sal_transport_available(Sal *sal, SalTransport t){
 	return FALSE;
 }
 
-int sal_add_listen_port(Sal *ctx, SalAddress* addr){
+static int sal_add_listen_port(Sal *ctx, SalAddress* addr, bool_t is_tunneled){
 	int result;
-	belle_sip_listening_point_t* lp = belle_sip_stack_create_listening_point(ctx->stack
-																			,sal_address_get_domain(addr)
-																			,sal_address_get_port(addr)
-																			,sal_transport_to_string(sal_address_get_transport(addr)));
+	belle_sip_listening_point_t* lp;
+	if (is_tunneled){
+#ifdef TUNNEL_ENABLED
+		if (sal_address_get_transport(addr)!=SalTransportUDP){
+			ms_error("Tunneled mode is only available for UDP kind of transports.");
+			return -1;
+		}
+		lp = belle_sip_tunnel_listening_point_new(ctx->stack, ctx->tunnel_client);
+		if (!lp){
+			ms_error("Could not create tunnel listening point.");
+			return -1;
+		}
+#else
+		ms_error("No tunnel support in library.");
+		return -1;
+#endif
+	}else{
+		lp = belle_sip_stack_create_listening_point(ctx->stack,
+									sal_address_get_domain(addr),
+									sal_address_get_port(addr),
+									sal_transport_to_string(sal_address_get_transport(addr)));
+	}
 	if (lp) {
 		belle_sip_listening_point_set_keep_alive(lp,ctx->keep_alive);
 		result = belle_sip_provider_add_listening_point(ctx->prov,lp);
-		set_tls_properties(ctx);
+		if (sal_address_get_transport(addr)==SalTransportTLS) set_tls_properties(ctx);
 	} else {
 		return -1;
 	}
 	return result;
 }
 
-int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int is_secure) {
+int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int is_tunneled) {
 	SalAddress* sal_addr = sal_address_new(NULL);
 	int result;
 	sal_address_set_domain(sal_addr,addr);
 	sal_address_set_port(sal_addr,port);
 	sal_address_set_transport(sal_addr,tr);
-	result = sal_add_listen_port(ctx,sal_addr);
+	result = sal_add_listen_port(ctx, sal_addr, is_tunneled);
 	sal_address_destroy(sal_addr);
 	return result;
 }
+
 static void remove_listening_point(belle_sip_listening_point_t* lp,belle_sip_provider_t* prov) {
 	belle_sip_provider_remove_listening_point(prov,lp);
 }
+
+int sal_get_listening_port(Sal *ctx, SalTransport tr){
+	const char *tpn=sal_transport_to_string(tr);
+	belle_sip_listening_point_t *lp=belle_sip_provider_get_listening_point(ctx->prov, tpn);
+	if (lp){
+		return belle_sip_listening_point_get_port(lp);
+	}
+	return 0;
+}
+
 int sal_unlisten_ports(Sal *ctx){
 	const belle_sip_list_t * lps = belle_sip_provider_get_listening_points(ctx->prov);
 	belle_sip_list_t * tmp_list = belle_sip_list_copy(lps);
 	belle_sip_list_for_each2 (tmp_list,(void (*)(void*,void*))remove_listening_point,ctx->prov);
 	belle_sip_list_free(tmp_list);
-
 	ms_message("sal_unlisten_ports done");
 	return 0;
 }
+
 ortp_socket_t sal_get_socket(Sal *ctx){
 	ms_warning("sal_get_socket is deprecated");
 	return -1;
 }
+
 void sal_set_user_agent(Sal *ctx, const char *user_agent){
 	belle_sip_header_user_agent_set_products(ctx->user_agent,NULL);
 	belle_sip_header_user_agent_add_product(ctx->user_agent,user_agent);
 	return ;
 }
 
+const char* sal_get_user_agent(Sal *ctx){
+	static char user_agent[255];
+	belle_sip_header_user_agent_get_products_as_string(ctx->user_agent, user_agent, 254);
+	return user_agent;
+}
+
 void sal_append_stack_string_to_user_agent(Sal *ctx) {
 	char stack_string[64];
 	snprintf(stack_string, sizeof(stack_string) - 1, "(belle-sip/%s)", belle_sip_version_to_string());
@@ -597,30 +668,16 @@ void sal_set_keepalive_period(Sal *ctx,unsigned int value){
 			belle_sip_listening_point_set_keep_alive(lp,ctx->keep_alive);
 		}
 	}
-	return ;
 }
-int sal_enable_tunnel(Sal *ctx, void *tunnelclient) {
+int sal_set_tunnel(Sal *ctx, void *tunnelclient) {
 #ifdef TUNNEL_ENABLED
-	belle_sip_listening_point_t *lp;
-	int result;
-
-	sal_unlisten_ports(ctx);
-	lp = belle_sip_tunnel_listening_point_new(ctx->stack, tunnelclient);
-	if (lp == NULL) return -1;
-
-	belle_sip_listening_point_set_keep_alive(lp, ctx->keep_alive);
-	result = belle_sip_provider_add_listening_point(ctx->prov, lp);
-	set_tls_properties(ctx);
-	return result;
-#else
+	ctx->tunnel_client=tunnelclient;
 	return 0;
+#else
+	return -1;
 #endif
 }
-void sal_disable_tunnel(Sal *ctx) {
-#ifdef TUNNEL_ENABLED
-	sal_unlisten_ports(ctx);
-#endif
-}
+
 /**
  * returns keepalive period in ms
  * 0 desactiaved
@@ -648,10 +705,10 @@ static void set_tls_properties(Sal *ctx){
 	if (lp){
 		belle_sip_tls_listening_point_t *tlp=BELLE_SIP_TLS_LISTENING_POINT(lp);
 		int verify_exceptions=0;
-		
+
 		if (!ctx->tls_verify) verify_exceptions=BELLE_SIP_TLS_LISTENING_POINT_BADCERT_ANY_REASON;
 		else if (!ctx->tls_verify_cn) verify_exceptions=BELLE_SIP_TLS_LISTENING_POINT_BADCERT_CN_MISMATCH;
-		
+
 		belle_sip_tls_listening_point_set_root_ca(tlp,ctx->root_ca); /*root_ca might be NULL */
 		belle_sip_tls_listening_point_set_verify_exceptions(tlp,verify_exceptions);
 	}
@@ -773,7 +830,7 @@ void sal_auth_info_set_mode(SalAuthInfo* auth_info, SalAuthMode mode) { auth_inf
 void sal_certificates_chain_delete(SalCertificatesChain *chain) {
 	belle_sip_object_unref((belle_sip_object_t *)chain);
 }
-void sal_signing_key_delete(SalSigningKey *key) { 
+void sal_signing_key_delete(SalSigningKey *key) {
 	belle_sip_object_unref((belle_sip_object_t *)key);
 }
 
@@ -800,7 +857,7 @@ int sal_auth_compute_ha1(const char* userid,const char* realm,const char* passwo
 SalCustomHeader *sal_custom_header_append(SalCustomHeader *ch, const char *name, const char *value){
 	belle_sip_message_t *msg=(belle_sip_message_t*)ch;
 	belle_sip_header_t *h;
-	
+
 	if (msg==NULL){
 		msg=(belle_sip_message_t*)belle_sip_request_new();
 		belle_sip_object_ref(msg);
@@ -817,7 +874,7 @@ SalCustomHeader *sal_custom_header_append(SalCustomHeader *ch, const char *name,
 const char *sal_custom_header_find(const SalCustomHeader *ch, const char *name){
 	if (ch){
 		belle_sip_header_t *h=belle_sip_message_get_header((belle_sip_message_t*)ch,name);
-		
+
 		if (h){
 			return belle_sip_header_get_unparsed_value(h);
 		}
@@ -850,12 +907,12 @@ void sal_set_uuid(Sal *sal, const char *uuid){
 }
 
 typedef struct {
-    unsigned int time_low;
-    unsigned short time_mid;
-    unsigned short time_hi_and_version;
-    unsigned char clock_seq_hi_and_reserved;
-    unsigned char clock_seq_low;
-    unsigned char node[6];
+	unsigned int time_low;
+	unsigned short time_mid;
+	unsigned short time_hi_and_version;
+	unsigned char clock_seq_hi_and_reserved;
+	unsigned char clock_seq_low;
+	unsigned char node[6];
 } sal_uuid_t;
 
 
@@ -863,7 +920,7 @@ int sal_create_uuid(Sal*ctx, char *uuid, size_t len){
 	sal_uuid_t uuid_struct;
 	int i;
 	int written;
-	
+
 	if (len==0) return -1;
 	/*create an UUID as described in RFC4122, 4.4 */
 	belle_sip_random_bytes((unsigned char*)&uuid_struct, sizeof(sal_uuid_t));
@@ -871,7 +928,7 @@ int sal_create_uuid(Sal*ctx, char *uuid, size_t len){
 	uuid_struct.clock_seq_hi_and_reserved|=1<<7;
 	uuid_struct.time_hi_and_version&=~(0xf<<12);
 	uuid_struct.time_hi_and_version|=4<<12;
-	
+
 	written=snprintf(uuid,len,"%8.8x-%4.4x-%4.4x-%2.2x%2.2x-", uuid_struct.time_low, uuid_struct.time_mid,
 			uuid_struct.time_hi_and_version, uuid_struct.clock_seq_hi_and_reserved,
 			uuid_struct.clock_seq_low);
@@ -886,10 +943,79 @@ int sal_create_uuid(Sal*ctx, char *uuid, size_t len){
 	return 0;
 }
 
+static void make_supported_header(Sal *sal){
+	MSList *it;
+	char *alltags=NULL;
+	size_t buflen=64;
+	size_t written=0;
+
+	if (sal->supported){
+		belle_sip_object_unref(sal->supported);
+		sal->supported=NULL;
+	}
+	for(it=sal->supported_tags;it!=NULL;it=it->next){
+		const char *tag=(const char*)it->data;
+		size_t taglen=strlen(tag);
+		if (alltags==NULL || (written+taglen+1>=buflen)) alltags=ms_realloc(alltags,(buflen=buflen*2));
+		snprintf(alltags+written,buflen-written,it->next ? "%s, " : "%s",tag);
+	}
+	if (alltags){
+		sal->supported=belle_sip_header_create("Supported",alltags);
+		if (sal->supported){
+			belle_sip_object_ref(sal->supported);
+		}
+		ms_free(alltags);
+	}
+}
+
+void sal_set_supported_tags(Sal *ctx, const char* tags){
+	ctx->supported_tags=ms_list_free_with_data(ctx->supported_tags,ms_free);
+	if (tags){
+		char *iter;
+		char *buffer=ms_strdup(tags);
+		char *tag;
+		char *context=NULL;
+		iter=buffer;
+		while((tag=strtok_r(iter,", ",&context))!=NULL){
+			iter=NULL;
+			ctx->supported_tags=ms_list_append(ctx->supported_tags,ms_strdup(tag));
+		}
+		ms_free(buffer);
+	}
+	make_supported_header(ctx);
+}
+
+const char *sal_get_supported_tags(Sal *ctx){
+	if (ctx->supported){
+		return belle_sip_header_get_unparsed_value(ctx->supported);
+	}
+	return NULL;
+}
+
+void sal_add_supported_tag(Sal *ctx, const char* tag){
+	MSList *elem=ms_list_find_custom(ctx->supported_tags,(MSCompareFunc)strcasecmp,tag);
+	if (!elem){
+		ctx->supported_tags=ms_list_append(ctx->supported_tags,ms_strdup(tag));
+		make_supported_header(ctx);
+	}
+
+}
+
+void sal_remove_supported_tag(Sal *ctx, const char* tag){
+	MSList *elem=ms_list_find_custom(ctx->supported_tags,(MSCompareFunc)strcasecmp,tag);
+	if (elem){
+		ms_free(elem->data);
+		ctx->supported_tags=ms_list_remove_link(ctx->supported_tags,elem);
+		make_supported_header(ctx);
+	}
+}
+
+
+
 belle_sip_response_t* sal_create_response_from_request ( Sal* sal, belle_sip_request_t* req, int code ) {
 	belle_sip_response_t *resp=belle_sip_response_create_from_request(req,code);
 	belle_sip_message_add_header(BELLE_SIP_MESSAGE(resp),BELLE_SIP_HEADER(sal->user_agent));
-	belle_sip_message_add_header(BELLE_SIP_MESSAGE(resp),sal_make_supported_header(sal));
+	belle_sip_message_add_header(BELLE_SIP_MESSAGE(resp),sal->supported);
 	return resp;
 }
 
@@ -909,6 +1035,10 @@ void sal_enable_test_features(Sal*ctx, bool_t enabled){
 	ctx->enable_test_features=enabled;
 }
 
+void sal_use_no_initial_route(Sal *ctx, bool_t enabled){
+	ctx->no_initial_route=enabled;
+}
+
 SalResolverContext * sal_resolve_a(Sal* sal, const char *name, int port, int family, SalResolverCallback cb, void *data){
 	return (SalResolverContext*)belle_sip_stack_resolve_a(sal->stack,name,port,family,(belle_sip_resolver_callback_t)cb,data);
 }
@@ -943,10 +1073,59 @@ void sal_signing_key_parse_file(SalAuthInfo* auth_info, const char* path, const
 	if (auth_info->key) belle_sip_object_ref((belle_sip_object_t *) auth_info->key);
 }
 
+/**
+ * Parse a directory to get a certificate with the given subject common name
+ *
+ */
+void sal_certificates_chain_parse_directory(char **certificate_pem, char **key_pem, char **fingerprint, const char* path, const char *subject, SalCertificateRawFormat format, bool_t generate_certificate, bool_t generate_dtls_fingerprint) {
+	belle_sip_certificates_chain_t *certificate = NULL;
+	belle_sip_signing_key_t *key = NULL;
+	*certificate_pem = NULL;
+	*key_pem = NULL;
+	if (belle_sip_get_certificate_and_pkey_in_dir(path, subject, &certificate, &key, (belle_sip_certificate_raw_format_t)format) == 0) {
+		*certificate_pem = belle_sip_certificates_chain_get_pem(certificate);
+		*key_pem = belle_sip_signing_key_get_pem(key);
+		ms_message("Retrieve certificate with CN=%s successful\n", subject);
+	} else {
+		if (generate_certificate == TRUE) {
+			if ( belle_sip_generate_self_signed_certificate(path, subject, &certificate, &key) == 0) {
+				*certificate_pem = belle_sip_certificates_chain_get_pem(certificate);
+				*key_pem = belle_sip_signing_key_get_pem(key);
+				ms_message("Generate self-signed certificate with CN=%s successful\n", subject);
+			}
+		}
+	}
+	/* generate the fingerprint as described in RFC4572 if needed */
+	if ((generate_dtls_fingerprint == TRUE) && (fingerprint != NULL)) {
+		if (*fingerprint != NULL) {
+			ms_free(*fingerprint);
+		}
+		*fingerprint = belle_sip_certificates_chain_get_fingerprint(certificate);
+	}
+
+	/* free key and certificate */
+	if ( certificate != NULL ) {
+		belle_sip_object_unref(certificate);
+	}
+	if ( key != NULL ) {
+		belle_sip_object_unref(key);
+	}
+}
+
 unsigned char * sal_get_random_bytes(unsigned char *ret, size_t size){
 	return belle_sip_random_bytes(ret,size);
 }
 
+char *sal_get_random_token(int size){
+	return belle_sip_random_token(ms_malloc(size),size);
+}
+
+unsigned int sal_get_random(void){
+	unsigned int ret=0;
+	belle_sip_random_bytes((unsigned char*)&ret,4);
+	return ret;
+}
+
 belle_sip_source_t * sal_create_timer(Sal *sal, belle_sip_source_func_t func, void *data, unsigned int timeout_value_ms, const char* timer_name) {
 	belle_sip_main_loop_t *ml = belle_sip_stack_get_main_loop(sal->stack);
 	return belle_sip_main_loop_create_timeout(ml, func, data, timeout_value_ms, timer_name);
@@ -956,3 +1135,21 @@ void sal_cancel_timer(Sal *sal, belle_sip_source_t *timer) {
 	belle_sip_main_loop_t *ml = belle_sip_stack_get_main_loop(sal->stack);
 	belle_sip_main_loop_remove_source(ml, timer);
 }
+
+unsigned long sal_begin_background_task(const char *name, void (*max_time_reached)(void *), void *data){
+	return belle_sip_begin_background_task(name, max_time_reached, data);
+}
+
+void sal_end_background_task(unsigned long id){
+	belle_sip_end_background_task(id);
+}
+
+
+void sal_enable_sip_update_method(Sal *ctx,bool_t value) {
+	ctx->enable_sip_update=value;
+}
+
+void sal_default_set_sdp_handling(Sal *sal, SalOpSDPHandling sdp_handling_method)  {
+	if (sdp_handling_method != SalOpSDPNormal ) ms_message("Enabling special SDP handling for all new SalOp in Sal[%p]!", sal);
+	sal->default_sdp_handling = sdp_handling_method;
+}
