@@ -41,6 +41,7 @@ static void conference_check_init(LinphoneConference *ctx, int samplerate){
 		MSAudioConferenceParams params;
 		params.samplerate=samplerate;
 		ctx->conf=ms_audio_conference_new(&params);
+		ctx->terminated=FALSE;
 	}
 }
 
@@ -74,7 +75,7 @@ void linphone_core_conference_check_uninit(LinphoneCore *lc){
 	if (ctx->conf){
 		int remote_count=remote_participants_count(ctx);
 		ms_message("conference_check_uninit(): size=%i",linphone_conference_get_size(ctx));
-		if (remote_count==1){
+		if (remote_count==1 && !ctx->terminated){
 			convert_conference_to_call(lc);
 		}
 		if (remote_count==0){
@@ -98,8 +99,8 @@ void linphone_call_add_to_conf(LinphoneCall *call, bool_t muted){
 	LinphoneCore *lc=call->core;
 	LinphoneConference *conf=&lc->conf_ctx;
 	MSAudioEndpoint *ep;
-	call->params.has_video = FALSE;
-	call->camera_active = FALSE;
+	call->params->has_video = FALSE;
+	call->camera_enabled = FALSE;
 	ep=ms_audio_endpoint_get_from_stream(call->audiostream,TRUE);
 	ms_audio_conference_add_member(conf->conf,ep);
 	ms_audio_conference_mute_member(conf->conf,ep,muted);
@@ -156,7 +157,7 @@ static void add_local_endpoint(LinphoneConference *conf,LinphoneCore *lc){
 /**
  * Returns the sound volume (mic input) of the local participant of the conference.
  * @param lc the linphone core
- * @returns the measured input volume expressed in dbm0.
+ * @return the measured input volume expressed in dbm0.
  **/
 float linphone_core_get_conference_local_input_volume(LinphoneCore *lc){
 	LinphoneConference *conf=&lc->conf_ctx;
@@ -179,20 +180,20 @@ float linphone_core_get_conference_local_input_volume(LinphoneCore *lc){
  * If the local user was actively part of the call (ie not in paused state), then the local user is automatically entered into the conference.
  * If the call was in paused state, then it is automatically resumed when entering into the conference.
  * 
- * @returns 0 if successful, -1 otherwise.
+ * @return 0 if successful, -1 otherwise.
 **/
 int linphone_core_add_to_conference(LinphoneCore *lc, LinphoneCall *call){
 	LinphoneConference *conf=&lc->conf_ctx;
 	
-	if (call->current_params.in_conference){
+	if (call->current_params->in_conference){
 		ms_error("Already in conference");
 		return -1;
 	}
 	conference_check_init(&lc->conf_ctx, lp_config_get_int(lc->config, "sound","conference_rate",16000));
 	
 	if (call->state==LinphoneCallPaused){
-		call->params.in_conference=TRUE;
-		call->params.has_video=FALSE;
+		call->params->in_conference=TRUE;
+		call->params->has_video=FALSE;
 		linphone_core_resume_call(lc,call);
 	}else if (call->state==LinphoneCallStreamsRunning){
 		LinphoneCallParams *params=linphone_call_params_copy(linphone_call_get_current_params(call));
@@ -200,12 +201,14 @@ int linphone_core_add_to_conference(LinphoneCore *lc, LinphoneCall *call){
 		params->has_video=FALSE;
 		
 		if (call->audiostream || call->videostream){
-			linphone_call_stop_media_streams (call); /*free the audio & video local resources*/
+			linphone_call_stop_media_streams(call); /*free the audio & video local resources*/
+			linphone_call_init_media_streams(call);
 		}
 		if (call==lc->current_call){
 			lc->current_call=NULL;
 		}
 		/*this will trigger a reINVITE that will later redraw the streams */
+		/*FIXME probably a bit too much to just redraw streams !*/
 		linphone_core_update_call(lc,call,params);
 		linphone_call_params_destroy(params);
 		add_local_endpoint(conf,lc);
@@ -218,9 +221,10 @@ int linphone_core_add_to_conference(LinphoneCore *lc, LinphoneCall *call){
 
 static int remove_from_conference(LinphoneCore *lc, LinphoneCall *call, bool_t active){
 	int err=0;
+	char *str;
 
-	if (!call->current_params.in_conference){
-		if (call->params.in_conference){
+	if (!call->current_params->in_conference){
+		if (call->params->in_conference){
 			ms_warning("Not (yet) in conference, be patient");
 			return -1;
 		}else{
@@ -228,9 +232,9 @@ static int remove_from_conference(LinphoneCore *lc, LinphoneCall *call, bool_t a
 			return -1;
 		}
 	}
-	call->params.in_conference=FALSE;
+	call->params->in_conference=FALSE;
 
-	char *str=linphone_call_get_remote_address_as_string(call);
+	str=linphone_call_get_remote_address_as_string(call);
 	ms_message("%s will be removed from conference", str);
 	ms_free(str);
 	if (active){
@@ -248,7 +252,6 @@ static int remove_from_conference(LinphoneCore *lc, LinphoneCall *call, bool_t a
 		ms_message("Pausing call to actually remove from conference");
 		err=_linphone_core_pause_call(lc,call);
 	}
-
 	return err;
 }
 
@@ -264,7 +267,7 @@ static int convert_conference_to_call(LinphoneCore *lc){
 	while (calls) {
 		LinphoneCall *rc=(LinphoneCall*)calls->data;
 		calls=calls->next;
-		if (rc->params.in_conference) { // not using current_param
+		if (rc->params->in_conference) { // not using current_param
 			bool_t active_after_removed=linphone_core_is_in_conference(lc);
 			err=remove_from_conference(lc, rc, active_after_removed);
 			break;
@@ -286,13 +289,14 @@ static int convert_conference_to_call(LinphoneCore *lc){
  * In other words, unless linphone_core_leave_conference() is explicitely called, the last remote participant of a conference is automatically
  * put in a simple call in running state.
  * 
- * @returns 0 if successful, -1 otherwise.
+ * @return 0 if successful, -1 otherwise.
  **/
 int linphone_core_remove_from_conference(LinphoneCore *lc, LinphoneCall *call){
+	int err;
 	char * str=linphone_call_get_remote_address_as_string(call);
 	ms_message("Removing call %s from the conference", str);
 	ms_free(str);
-	int err=remove_from_conference(lc,call, FALSE);
+	err=remove_from_conference(lc,call, FALSE);
 	if (err){
 		ms_error("Error removing participant from conference.");
 		return err;
@@ -310,7 +314,7 @@ int linphone_core_remove_from_conference(LinphoneCore *lc, LinphoneCall *call){
 /**
  * Indicates whether the local participant is part of the conference.
  * @param lc the linphone core
- * @returns TRUE if the local participant is in the conference, FALSE otherwise.
+ * @return TRUE if the local participant is in the conference, FALSE otherwise.
 **/
 bool_t linphone_core_is_in_conference(const LinphoneCore *lc){
 	return lc->conf_ctx.local_participant!=NULL;
@@ -320,7 +324,7 @@ bool_t linphone_core_is_in_conference(const LinphoneCore *lc){
  * Moves the local participant out of the conference.
  * @param lc the linphone core
  * When the local participant is out of the conference, the remote participants can continue to talk normally.
- * @returns 0 if successful, -1 otherwise.
+ * @return 0 if successful, -1 otherwise.
 **/
 int linphone_core_leave_conference(LinphoneCore *lc){
 	LinphoneConference *conf=&lc->conf_ctx;
@@ -338,16 +342,17 @@ int linphone_core_leave_conference(LinphoneCore *lc){
  * However, by calling linphone_core_leave_conference() and linphone_core_enter_conference() the application can decide to temporarily
  * move out and in the local participant from the conference.
  * 
- * @returns 0 if successful, -1 otherwise
+ * @return 0 if successful, -1 otherwise
 **/
 int linphone_core_enter_conference(LinphoneCore *lc){
+	LinphoneConference *conf;
 	if (linphone_core_sound_resources_locked(lc)) {
 		return -1;
 	}
 	if (lc->current_call != NULL) {
 		_linphone_core_pause_call(lc, lc->current_call);
 	}
-	LinphoneConference *conf=&lc->conf_ctx;
+	conf=&lc->conf_ctx;
 	if (conf->local_participant==NULL) add_local_endpoint(conf,lc);
 	return 0;
 }
@@ -358,14 +363,14 @@ int linphone_core_enter_conference(LinphoneCore *lc){
  * 
  * Merge all established calls (either in LinphoneCallStreamsRunning or LinphoneCallPaused) into a conference.
  * 
- * @returns 0 if successful, -1 otherwise
+ * @return 0 if successful, -1 otherwise
 **/
 int linphone_core_add_all_to_conference(LinphoneCore *lc) {
 	MSList *calls=lc->calls;
 	while (calls) {
 		LinphoneCall *call=(LinphoneCall*)calls->data;
 		calls=calls->next;
-		if (!call->current_params.in_conference) {
+		if (!call->current_params->in_conference) {
 			linphone_core_add_to_conference(lc, call);
 		}
 	}
@@ -379,14 +384,17 @@ int linphone_core_add_all_to_conference(LinphoneCore *lc) {
  * 
  * All the calls that were merged to the conference are terminated, and the conference resources are destroyed.
  * 
- * @returns 0 if successful, -1 otherwise
+ * @return 0 if successful, -1 otherwise
 **/
 int linphone_core_terminate_conference(LinphoneCore *lc) {
 	MSList *calls=lc->calls;
+	LinphoneConference *conf=&lc->conf_ctx;
+	conf->terminated=TRUE;
+
 	while (calls) {
 		LinphoneCall *call=(LinphoneCall*)calls->data;
 		calls=calls->next;
-		if (call->current_params.in_conference) {
+		if (call->current_params->in_conference) {
 			linphone_core_terminate_call(lc, call);
 		}
 	}
@@ -400,7 +408,7 @@ int linphone_core_terminate_conference(LinphoneCore *lc) {
  * Typically, after merging two calls into the conference, there is total of 3 participants:
  * the local participant (or local user), and two remote participants that were the destinations of the two previously establised calls.
  * 
- * @returns the number of participants to the conference
+ * @return the number of participants to the conference
 **/
 int linphone_core_get_conference_size(LinphoneCore *lc) {
 	LinphoneConference *conf=&lc->conf_ctx;
