@@ -154,7 +154,7 @@ static void ecc_play_tones(EcCalibrator *ecc){
 	memset(&tone,0,sizeof(tone));
 	memset(&expected_tone,0,sizeof(expected_tone));
 
-	ms_filter_set_notify_callback(ecc->det,on_tone_received,ecc);
+	ms_filter_add_notify_callback(ecc->det,on_tone_received,ecc,TRUE);
 
 	/* configure the tones to be scanned */
 	
@@ -188,7 +188,7 @@ static void ecc_play_tones(EcCalibrator *ecc){
 	ms_filter_call_method(ecc->gen,MS_DTMF_GEN_PLAY_CUSTOM,&tone);
 	ms_sleep(2);
 
-	ms_filter_set_notify_callback(ecc->gen,on_tone_sent,ecc);
+	ms_filter_add_notify_callback(ecc->gen,on_tone_sent,ecc,TRUE);
 	
 	/* play the three tones*/
 	
@@ -265,11 +265,13 @@ void ec_calibrator_destroy(EcCalibrator *ecc){
 
 int linphone_core_start_echo_calibration(LinphoneCore *lc, LinphoneEcCalibrationCallback cb,
 					 LinphoneEcCalibrationAudioInit audio_init_cb, LinphoneEcCalibrationAudioUninit audio_uninit_cb, void *cb_data){
+	unsigned int rate;
+
 	if (lc->ecc!=NULL){
 		ms_error("Echo calibration is still on going !");
 		return -1;
 	}
-	unsigned int rate = lp_config_get_int(lc->config,"sound","echo_cancellation_rate",8000);
+	rate = lp_config_get_int(lc->config,"sound","echo_cancellation_rate",8000);
 	lc->ecc=ec_calibrator_new(lc->sound_conf.play_sndcard,lc->sound_conf.capt_sndcard,rate,cb,audio_init_cb,audio_uninit_cb,cb_data);
 	return 0;
 }
