@@ -81,15 +81,17 @@ static const char *person_prefix = "/pidf:presence/dm:person";
 /*****************************************************************************
  * PRIVATE FUNCTIONS                                                         *
  ****************************************************************************/
-
-static char presence_id_valid_characters[] = "0123456789abcdefghijklmnopqrstuvwxyz";
+/*defined in http://www.w3.org/TR/REC-xml/*/
+static char presence_id_valid_characters[] = "0123456789abcdefghijklmnopqrstuvwxyz-.";
+/*NameStartChar (NameChar)**/
+static char presence_id_valid_start_characters[] = ":_abcdefghijklmnopqrstuvwxyz";
 
 static char * generate_presence_id(void) {
 	char id[7];
 	int i;
-
-	for (i = 0; i < 6; i++) {
-		id[i] = presence_id_valid_characters[random() % sizeof(presence_id_valid_characters)];
+	id[0] = presence_id_valid_start_characters[ortp_random() % (sizeof(presence_id_valid_start_characters)-1)];
+	for (i = 1; i < 6; i++) {
+		id[i] = presence_id_valid_characters[ortp_random() % (sizeof(presence_id_valid_characters)-1)];
 	}
 	id[6] = '\0';
 
@@ -170,7 +172,7 @@ static time_t parse_timestamp(const char *timestamp) {
 	return seconds - timezone;
 }
 
-static char * timestamp_to_string(time_t timestamp) {
+char * linphone_timestamp_to_rfc3339_string(time_t timestamp) {
 	char timestamp_str[22];
 	struct tm *ret;
 #ifndef WIN32
@@ -278,6 +280,7 @@ LinphonePresenceBasicStatus linphone_presence_model_get_basic_status(const Linph
 
 int linphone_presence_model_set_basic_status(LinphonePresenceModel *model, LinphonePresenceBasicStatus basic_status) {
 	LinphonePresenceService *service;
+	int err = 0;
 
 	if (model == NULL) return -1;
 
@@ -285,8 +288,9 @@ int linphone_presence_model_set_basic_status(LinphonePresenceModel *model, Linph
 	service = linphone_presence_service_new(NULL, basic_status, NULL);
 	if (service == NULL) return -1;
 
-	if (linphone_presence_model_add_service(model, service) < 0) return -1;
-	return 0;
+	err = linphone_presence_model_add_service(model, service);
+	linphone_presence_service_unref(service);
+	return err;
 }
 
 static void presence_service_find_newer_timestamp(LinphonePresenceService *service, time_t *timestamp) {
@@ -363,6 +367,7 @@ LinphonePresenceActivity * linphone_presence_model_get_activity(const LinphonePr
 int linphone_presence_model_set_activity(LinphonePresenceModel *model, LinphonePresenceActivityType acttype, const char *description) {
 	LinphonePresenceBasicStatus basic_status = LinphonePresenceBasicStatusOpen;
 	LinphonePresenceActivity *activity;
+	int err = 0;
 
 	if (model == NULL) return -1;
 
@@ -383,8 +388,9 @@ int linphone_presence_model_set_activity(LinphonePresenceModel *model, LinphoneP
 	linphone_presence_model_clear_activities(model);
 	activity = linphone_presence_activity_new(acttype, description);
 	if (activity == NULL) return -1;
-	return linphone_presence_model_add_activity(model, activity);
-
+	err = linphone_presence_model_add_activity(model, activity);
+	linphone_presence_activity_unref(activity);
+	return err;
 }
 
 unsigned int linphone_presence_model_get_nb_activities(const LinphonePresenceModel *model) {
@@ -1037,7 +1043,7 @@ void linphone_presence_model_set_user_data(LinphonePresenceModel *model, void *u
 	model->user_data = user_data;
 }
 
-void * linphone_presence_model_get_user_data(LinphonePresenceModel *model) {
+void * linphone_presence_model_get_user_data(const LinphonePresenceModel *model) {
 	return model->user_data;
 }
 
@@ -1059,7 +1065,7 @@ void linphone_presence_service_set_user_data(LinphonePresenceService *service, v
 	service->user_data = user_data;
 }
 
-void * linphone_presence_service_get_user_data(LinphonePresenceService *service) {
+void * linphone_presence_service_get_user_data(const LinphonePresenceService *service) {
 	return service->user_data;
 }
 
@@ -1081,7 +1087,7 @@ void linphone_presence_person_set_user_data(LinphonePresencePerson *person, void
 	person->user_data = user_data;
 }
 
-void * linphone_presence_person_get_user_data(LinphonePresencePerson *person) {
+void * linphone_presence_person_get_user_data(const LinphonePresencePerson *person) {
 	return person->user_data;
 }
 
@@ -1103,7 +1109,7 @@ void linphone_presence_activity_set_user_data(LinphonePresenceActivity *activity
 	activity->user_data = user_data;
 }
 
-void * linphone_presence_activity_get_user_data(LinphonePresenceActivity *activity) {
+void * linphone_presence_activity_get_user_data(const LinphonePresenceActivity *activity) {
 	return activity->user_data;
 }
 
@@ -1125,7 +1131,7 @@ void linphone_presence_note_set_user_data(LinphonePresenceNote *note, void *user
 	note->user_data = user_data;
 }
 
-void * linphone_presence_note_get_user_data(LinphonePresenceNote *note) {
+void * linphone_presence_note_get_user_data(const LinphonePresenceNote *note) {
 	return note->user_data;
 }
 
@@ -1205,19 +1211,15 @@ static int process_pidf_xml_presence_services(xmlparsing_context_t *xml_ctx, Lin
 			service_id_str = linphone_get_xml_text_content(xml_ctx, xpath_str);
 			service = presence_service_new(service_id_str, basic_status);
 			if (service != NULL) {
-				if (timestamp_str != NULL) {
-					presence_service_set_timestamp(service, parse_timestamp(timestamp_str));
-					linphone_free_xml_text_content(timestamp_str);
-				}
-				if (contact_str != NULL) {
-					linphone_presence_service_set_contact(service, contact_str);
-					linphone_free_xml_text_content(contact_str);
-				}
+				if (timestamp_str != NULL) presence_service_set_timestamp(service, parse_timestamp(timestamp_str));
+				if (contact_str != NULL) linphone_presence_service_set_contact(service, contact_str);
 				process_pidf_xml_presence_service_notes(xml_ctx, service, i);
 				linphone_presence_model_add_service(model, service);
 			}
-			linphone_free_xml_text_content(basic_status_str);
+			if (timestamp_str != NULL) linphone_free_xml_text_content(timestamp_str);
+			if (contact_str != NULL) linphone_free_xml_text_content(contact_str);
 			if (service_id_str != NULL) linphone_free_xml_text_content(service_id_str);
+			linphone_free_xml_text_content(basic_status_str);
 		}
 	}
 	if (service_object != NULL) xmlXPathFreeObject(service_object);
@@ -1437,9 +1439,9 @@ void linphone_core_add_subscriber(LinphoneCore *lc, const char *subscriber, SalO
 	linphone_friend_set_inc_subscribe_policy(fl,LinphoneSPAccept);
 	fl->inc_subscribe_pending=TRUE;
 	lc->subscribers=ms_list_append(lc->subscribers,(void *)fl);
-	if (lc->vtable.new_subscription_requested!=NULL) {
+	{
 		char *tmp=linphone_address_as_string(fl->uri);
-		lc->vtable.new_subscription_requested(lc,fl,tmp);
+		linphone_core_notify_new_subscription_requested(lc,fl,tmp);
 		ms_free(tmp);
 	}
 }
@@ -1467,7 +1469,7 @@ void linphone_subscription_new(LinphoneCore *lc, SalOp *op, const char *from){
 	char *tmp;
 	LinphoneAddress *uri;
 	LinphoneProxyConfig *cfg;
-	
+
 	uri=linphone_address_new(from);
 	linphone_address_clean(uri);
 	tmp=linphone_address_as_string(uri);
@@ -1482,7 +1484,7 @@ void linphone_subscription_new(LinphoneCore *lc, SalOp *op, const char *from){
 			}
 		}
 	}
-	
+
 	/* check if we answer to this subscription */
 	if (linphone_find_friend_by_address(lc->friends,uri,&lf)!=NULL){
 		lf->insub=op;
@@ -1604,7 +1606,7 @@ static void write_xml_presence_note_obj(LinphonePresenceNote *note, struct _pres
 
 static int write_xml_presence_timestamp(xmlTextWriterPtr writer, time_t timestamp) {
 	int err;
-	char *timestamp_str = timestamp_to_string(timestamp);
+	char *timestamp_str = linphone_timestamp_to_rfc3339_string(timestamp);
 	err = xmlTextWriterWriteElement(writer, (const xmlChar *)"timestamp", (const xmlChar *)timestamp_str);
 	if (timestamp_str) ms_free(timestamp_str);
 	return err;
@@ -1790,6 +1792,8 @@ void linphone_notify_convert_presence_to_xml(SalOp *op, SalPresenceModel *presen
 		return;
 	}
 
+	xmlTextWriterSetIndent(writer,1);
+
 	err = xmlTextWriterStartDocument(writer, "1.0", "UTF-8", NULL);
 	if (err >= 0) {
 		err = xmlTextWriterStartElementNS(writer, NULL, (const xmlChar *)"presence", (const xmlChar *)"urn:ietf:params:xml:ns:pidf");
@@ -1870,8 +1874,7 @@ void linphone_notify_recv(LinphoneCore *lc, SalOp *op, SalSubscribeStatus ss, Sa
 		}
 		lf->presence = presence;
 		lf->subscribe_active=TRUE;
-		if (lc->vtable.notify_presence_received)
-			lc->vtable.notify_presence_received(lc,(LinphoneFriend*)lf);
+		linphone_core_notify_notify_presence_received(lc,(LinphoneFriend*)lf);
 		ms_free(tmp);
 	}else{
 		ms_message("But this person is not part of our friend list, so we don't care.");
