@@ -132,7 +132,7 @@ static void _linphone_status_icon_desc_is_supported_result_cb(
 	_LinphoneStatusIconDescSearchCtx *ctx) {
 	
 	if(!result) {
-		;
+		ctx->i = g_slist_next(ctx->i);
 		for(; ctx->i; ctx->i = g_slist_next(ctx->i)) {
 			if(_linphone_status_icon_desc_is_supported(
 				(const _LinphoneStatusIconDesc *)g_slist_nth_data(ctx->i, 0),
@@ -297,17 +297,15 @@ static void _linphone_status_icon_impl_gtk_init(LinphoneStatusIcon *si) {
 	g_signal_connect_swapped(G_OBJECT(icon),"activate", G_CALLBACK(_linphone_status_icon_impl_gtk_on_click_cb), si);
 	g_signal_connect(G_OBJECT(icon), "popup-menu", G_CALLBACK(_linphone_status_icon_impl_gtk_popup_menu), si);
 	g_object_set_data_full(G_OBJECT(icon),"icon",pbuf, g_object_unref);
-	g_object_unref(pbuf);
 	pbuf=create_pixbuf(call_icon_path);
 	g_object_set_data_full(G_OBJECT(icon),"call_icon",pbuf, g_object_unref);
-	g_object_unref(pbuf);
 	si->data = icon;
 }
 
-static void _linphone_status_icon_impl_gtk_uninit(LinphoneStatusIcon *si) {
-	GtkStatusIcon *icon = GTK_STATUS_ICON(si->data);
-	gtk_status_icon_set_visible(icon, FALSE);
-}
+// static void _linphone_status_icon_impl_gtk_uninit(LinphoneStatusIcon *si) {
+// 	GtkStatusIcon *icon = GTK_STATUS_ICON(si->data);
+// 	gtk_status_icon_set_visible(icon, FALSE);
+// }
 
 static void _linphone_status_icon_impl_gtk_start(LinphoneStatusIcon *si) {
 	GtkStatusIcon *icon = GTK_STATUS_ICON(si->data);
@@ -359,7 +357,7 @@ static gboolean _linphone_status_icon_impl_is_supported(
 static const _LinphoneStatusIconDesc _linphone_status_icon_impl_gtk_desc = {
 	"gtk_status_icon",
 	_linphone_status_icon_impl_gtk_init,
-	_linphone_status_icon_impl_gtk_uninit,
+	NULL,
 	_linphone_status_icon_impl_gtk_start,
 	_linphone_status_icon_impl_enable_blinking,
 	_linphone_status_icon_impl_is_supported
@@ -412,9 +410,9 @@ static void _linphone_status_icon_impl_sn_init(LinphoneStatusIcon *si) {
 	si->data = bc_status_notifier_new();
 }
 
-static void _linphone_status_icon_impl_sn_uninit(LinphoneStatusIcon *si) {
-	bc_status_notifier_unref((BcStatusNotifier *)si->data);
-}
+// static void _linphone_status_icon_impl_sn_uninit(LinphoneStatusIcon *si) {
+// 	bc_status_notifier_unref((BcStatusNotifier *)si->data);
+// }
 
 static void _linphone_status_icon_impl_sn_activated_cb(BcStatusNotifier *sn, int x, int y, void *user_data) {
 	LinphoneStatusIcon *si = (LinphoneStatusIcon *)user_data;
@@ -490,10 +488,18 @@ static gboolean _linphone_status_icon_impl_sn_is_supported(
 	LinphoneStatusIconDescIsSupportedResultCb cb,
 	void *user_data) {
 	
-	_LinphoneStatusIconDesc *desc2 = g_new(_LinphoneStatusIconDesc, 1);
-	void **data = g_new(void *, 3);
+	_LinphoneStatusIconDesc *desc2;
+	void **data;
+	const char *desktop = g_getenv("XDG_CURRENT_DESKTOP");
 	
+	if(desktop == NULL || g_strcmp0(desktop, "KDE") != 0) {
+		*result = FALSE;
+		return TRUE;
+	}
+	
+	desc2 = g_new(_LinphoneStatusIconDesc, 1);
 	*desc2 = *desc;
+	data = g_new(void *, 3);
 	data[0] = desc2;
 	data[1] = cb;
 	data[2] = user_data;
@@ -502,13 +508,13 @@ static gboolean _linphone_status_icon_impl_sn_is_supported(
 		(BcStatusNotifierSupportDetectionCb)_linphone_status_icon_impl_is_supported_cb,
 		data
 	);
-	return 0;
+	return FALSE;
 }
 
 static const _LinphoneStatusIconDesc _linphone_status_icon_impl_status_notifier = {
 	.impl_name = "status_notifier",
 	.init = _linphone_status_icon_impl_sn_init,
-	.uninit = _linphone_status_icon_impl_sn_uninit,
+	.uninit = NULL,
 	.start = _linphone_status_icon_impl_sn_start,
 	.enable_blinking = _linphone_status_icon_impl_sn_enable_blinking,
 	.is_supported = _linphone_status_icon_impl_sn_is_supported
