@@ -16,15 +16,13 @@ else
 	AUTOMAKE=automake-${AM_VERSION}
 fi
 
-INTLTOOLIZE=/usr/bin/intltoolize
+INTLTOOLIZE=$(which intltoolize)
 
-if test -f /opt/local/bin/intltoolize ; then
-INTLTOOLIZE=/opt/local/bin/intltoolize
-else
-INTLTOOLIZE=/usr/bin/intltoolize
+#workaround for mingw bug in intltoolize script.
+if test "$INTLTOOLIZE" = "/bin/intltoolize" ; then
+	INTLTOOLIZE=/usr/bin/intltoolize
 fi
 
-
 libtoolize="libtoolize"
 for lt in glibtoolize libtoolize15 libtoolize14 libtoolize13 ; do
         if test -x /usr/bin/$lt ; then
