@@ -114,7 +114,7 @@ static void ms_opus_enc_preprocess(MSFilter *f) {
 	}
 
 	/* set complexity to 0 for single processor arm devices */
-#ifdef __arm__
+#if defined(__arm__) || defined(_M_ARM)
 	if (ms_factory_get_cpu_count(f->factory)==1){
 		opus_encoder_ctl(d->state, OPUS_SET_COMPLEXITY(0));
 	}else{
@@ -203,6 +203,7 @@ static void ms_opus_enc_process(MSFilter *f) {
 
 			om->b_wptr += ret;
 			mblk_set_timestamp_info(om, d->ts);
+			ms_bufferizer_fill_current_metas(d->bufferizer, om);
 			ms_queue_put(f->outputs[0], om);
 			d->ts += packet_size*48000/d->samplerate; /* RFC payload RTP opus 03 - section 4: RTP timestamp multiplier : WARNING works only with sr at 48000 */
 			ret = 0;
@@ -796,6 +797,12 @@ static int ms_opus_dec_set_nchannels(MSFilter *f, void *arg) {
 	return 0;
 }
 
+static int ms_opus_dec_get_nchannels(MSFilter *f, void *arg) {
+	OpusDecData *d = (OpusDecData *)f->data;
+	*(int*)arg=d->channels;
+	return 0;
+}
+
 static MSFilterMethod ms_opus_dec_methods[] = {
 	{	MS_FILTER_SET_SAMPLE_RATE,	ms_opus_dec_set_sample_rate	},
 	{	MS_FILTER_GET_SAMPLE_RATE,	ms_opus_dec_get_sample_rate	},
@@ -804,6 +811,7 @@ static MSFilterMethod ms_opus_dec_methods[] = {
 	{	MS_AUDIO_DECODER_SET_RTP_PAYLOAD_PICKER, ms_opus_set_rtp_picker },
 	{ 	MS_AUDIO_DECODER_HAVE_PLC,		ms_opus_dec_have_plc	},
 	{	MS_FILTER_SET_NCHANNELS,	ms_opus_dec_set_nchannels	},
+	{	MS_FILTER_GET_NCHANNELS,	ms_opus_dec_get_nchannels	},
 	{	0,				NULL				}
 };
 
