@@ -139,6 +139,7 @@ typedef struct _DDDisplay{
 	bool_t autofit;
 	bool_t mirroring;
 	bool_t own_window;
+	bool_t auto_window;
 }DDDisplay;
 
 static LRESULT CALLBACK window_proc(
@@ -149,6 +150,10 @@ static LRESULT CALLBACK window_proc(
 {
 	DDDisplay *wd=(DDDisplay*)GetWindowLongPtr(hwnd,GWLP_USERDATA);
 	switch(uMsg){
+		case WM_CREATE:
+			wd=(DDDisplay*)((LPCREATESTRUCT)lParam)->lpCreateParams;
+			SetWindowLongPtr(hwnd,GWL_USERDATA,(long)wd);
+			break;
 		case WM_DESTROY:
 			if (wd){
 				wd->window=NULL;
@@ -180,7 +185,7 @@ static LRESULT CALLBACK window_proc(
 	return 0;
 }
 
-static HWND create_window(int w, int h)
+static HWND create_window(int w, int h, DDDisplay *dd)
 {
 	WNDCLASS wc;
 	HINSTANCE hInstance = GetModuleHandle(NULL);
@@ -212,7 +217,7 @@ static HWND create_window(int w, int h)
 	hwnd=CreateWindow("Video Window", "Video window", 
 		WS_OVERLAPPEDWINDOW /*WS_THICKFRAME*/ | WS_VISIBLE ,
 		CW_USEDEFAULT, CW_USEDEFAULT, rect.right-rect.left,rect.bottom-rect.top,
-													NULL, NULL, hInstance, NULL);
+													NULL, NULL, hInstance, dd);
 	if (hwnd==NULL){
 		ms_error("Fail to create video window");
 	}
@@ -237,15 +242,17 @@ static void dd_display_init(MSFilter  *f){
 	obj->autofit=TRUE;
 	obj->mirroring=FALSE;
 	obj->own_window=TRUE;
+	obj->auto_window=TRUE;
 	f->data=obj;
 }
 
 static void dd_display_prepare(MSFilter *f){
 	DDDisplay *dd=(DDDisplay*)f->data;
 	
-	if (dd->window==NULL){
-		dd->window=create_window(dd->wsize.width,dd->wsize.height);
-		SetWindowLong(dd->window,GWL_USERDATA,(long)dd);
+	if (dd->window==NULL) {
+		if (dd->auto_window) {
+			dd->window=create_window(dd->wsize.width,dd->wsize.height,dd);
+		}
 	}
 	if (dd->ddh==NULL)
 		dd->ddh=DrawDibOpen();
@@ -278,7 +285,7 @@ static void dd_display_uninit(MSFilter *f){
 }
 
 static void dd_display_preprocess(MSFilter *f){
-	dd_display_prepare(f);
+	
 }
 
 
@@ -369,6 +376,12 @@ static void dd_display_process(MSFilter *f){
 	int corner=obj->sv_corner;
 	float scalefactor=obj->sv_scalefactor;
 
+	/* this creates the window if not given by the application. This must be done within the process() function because
+	 it is not possible on windows to create a windows in one thread and modify it from another one.
+	 Previously, window creation was done from preprocess() (so main thread) but it was deadlocking when process() function
+	 was doing MoveWindow() to resize the window*/
+	dd_display_prepare(f);
+	
 	if (obj->window==NULL){
 		goto end;
 	}
@@ -378,6 +391,8 @@ static void dd_display_process(MSFilter *f){
 
 	wsize.width=rect.right;
 	wsize.height=rect.bottom;
+	if (!ms_video_size_equal(wsize,obj->wsize))
+		obj->need_repaint=TRUE;
 	obj->wsize=wsize;
 	/*get most recent message and draw it*/
 	if (corner!=-1 && f->inputs[1]!=NULL && (local_im=ms_queue_peek_last(f->inputs[1]))!=NULL) {
@@ -389,14 +404,16 @@ static void dd_display_process(MSFilter *f){
 	
 	if (f->inputs[0]!=NULL && (main_im=ms_queue_peek_last(f->inputs[0]))!=NULL) {
 		if (ms_yuv_buf_init_from_mblk(&mainpic,main_im)==0){
-			if (obj->autofit && (obj->vsize.width!=mainpic.w || obj->vsize.height!=mainpic.h)
-				&& (mainpic.w>wsize.width || mainpic.h>wsize.height)){
-				RECT cur;
-				ms_message("Detected video resolution changed, resizing window");
-				GetWindowRect(obj->window,&cur);
-				wsize.width=mainpic.w;
-				wsize.height=mainpic.h;
-				MoveWindow(obj->window,cur.left, cur.top, wsize.width, wsize.height,TRUE);
+			if (obj->vsize.width!=mainpic.w || obj->vsize.height!=mainpic.h){
+				ms_message("Detected video resolution changed to %ix%i",mainpic.w,mainpic.h);
+				if (obj->autofit && (mainpic.w>wsize.width || mainpic.h>wsize.height) ){
+					RECT cur;
+					GetWindowRect(obj->window,&cur);
+					wsize.width=mainpic.w;
+					wsize.height=mainpic.h;
+					MoveWindow(obj->window,cur.left, cur.top, wsize.width, wsize.height,TRUE);
+				}
+				//in all case repaint the background.
 				obj->need_repaint=TRUE;
 			}
 			obj->vsize.width=mainpic.w;
@@ -472,7 +489,13 @@ static void dd_display_process(MSFilter *f){
 	}
 	
 	end:
-		
+	{
+		MSG msg;
+		while (PeekMessage(&msg, NULL, 0, 0,1)){
+			TranslateMessage(&msg);
+			DispatchMessage(&msg);
+		}
+	}
 	if (f->inputs[0]!=NULL)
 		ms_queue_flush(f->inputs[0]);
 	if (f->inputs[1]!=NULL)
@@ -481,14 +504,25 @@ static void dd_display_process(MSFilter *f){
 
 static int get_native_window_id(MSFilter *f, void *data){
 	DDDisplay *obj=(DDDisplay*)f->data;
-	*(long*)data=(long)obj->window;
+	if(obj->auto_window) {
+		*(long*)data=(long)obj->window;
+	} else {
+		*(unsigned long*)data=MS_FILTER_VIDEO_NONE;
+	}
 	return 0;
 }
 
 static int set_native_window_id(MSFilter *f, void *data){
 	DDDisplay *obj=(DDDisplay*)f->data;
-	obj->window=(HWND)(*(long*)data);
-	obj->own_window=FALSE;
+	unsigned long winId = *((unsigned long*)data);
+	if(winId != MS_FILTER_VIDEO_NONE) {
+		obj->window=(HWND)(*(long*)data);
+		obj->own_window=FALSE;
+		obj->auto_window=TRUE;
+	} else {
+		obj->window=NULL;
+		obj->auto_window=FALSE;
+	}
 	return 0;
 }
 
