@@ -31,15 +31,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 struct v4mState;
 
 // Define != NULL to have QT Framework convert hardware device pixel format to another one.
-#if TARGET_CPU_X86 /*for an unknown reason kCVPixelFormatType_420YpCbCr8Planar does not work for 32bits compilation*/
-static OSType forcedPixelFormat=kCVPixelFormatType_24RGB;
-#else
-static OSType forcedPixelFormat=kCVPixelFormatType_420YpCbCr8Planar;
-#endif
+static OSType forcedPixelFormat=kCVPixelFormatType_422YpCbCr8_yuvs;
 //static OSType forcedPixelFormat=0;
 
 static MSPixFmt ostype_to_pix_fmt(OSType pixelFormat, bool printFmtName){
-	// ms_message("OSType= %i", pixelFormat);
+        ms_message("OSType= %i", (int)pixelFormat);
         switch(pixelFormat){
                 case kCVPixelFormatType_420YpCbCr8Planar:
                 	if (printFmtName) ms_message("FORMAT = MS_YUV420P");
@@ -97,8 +93,7 @@ static MSPixFmt ostype_to_pix_fmt(OSType pixelFormat, bool printFmtName){
 {
 	ms_mutex_lock(&mutex);	
 
-    	OSType pixelFormat = CVPixelBufferGetPixelFormatType(frame);
-        MSPixFmt msfmt = ostype_to_pix_fmt(pixelFormat, false);
+    	//OSType pixelFormat = CVPixelBufferGetPixelFormatType(frame);
 
 	if (CVPixelBufferIsPlanar(frame)) {
 		size_t numberOfPlanes = CVPixelBufferGetPlaneCount(frame);
@@ -211,11 +206,11 @@ static MSPixFmt ostype_to_pix_fmt(OSType pixelFormat, bool printFmtName){
 				if (msfmt != MS_PIX_FMT_UNKNOWN) {
 					return msfmt;
 				}
-            		}
-        	}
-    	} else {
-    		ms_error("The camera wasn't opened when asking for pixel format");
-    	}
+            }
+        }
+    } else {
+        ms_error("The camera wasn't opened when asking for pixel format");
+    }
 
 	ms_warning("No compatible format found, using MS_YUV420P pixel format");
 	// Configure the output to convert the uncompatible hardware pixel format to MS_YUV420P
@@ -272,7 +267,7 @@ static MSPixFmt ostype_to_pix_fmt(OSType pixelFormat, bool printFmtName){
 - (void)setSize:(MSVideoSize)size {	
 	NSDictionary *dic;
 	if (forcedPixelFormat != 0) {
-		ms_message("QTCapture set size w=%i, h=%i fmt=%i", size.width, size.height, forcedPixelFormat);
+		ms_message("QTCapture set size w=%i, h=%i fmt=%i", size.width, size.height, (unsigned int)forcedPixelFormat);
 		dic = [NSDictionary dictionaryWithObjectsAndKeys:
 		 [NSNumber numberWithInteger:size.width], (id)kCVPixelBufferWidthKey,
 		 [NSNumber numberWithInteger:size.height],(id)kCVPixelBufferHeightKey,
@@ -322,6 +317,7 @@ typedef struct v4mState {
 	float fps;
 	float start_time;
 	int frame_count;
+	MSAverageFPS afps;
 } v4mState;
 
 
@@ -392,6 +388,7 @@ static void v4m_process(MSFilter * obj){
 			mblk_set_marker_info(om,TRUE);	
 			ms_queue_put(obj->outputs[0],om);
 			s->frame_count++;
+			ms_average_fps_update(&s->afps, obj->ticker->time);
 		}
 	} else {
 		flushq([s->webcam rq],0);
@@ -403,12 +400,15 @@ static void v4m_process(MSFilter * obj){
 }
 
 static void v4m_preprocess(MSFilter *f) {
-	v4m_start(f,NULL);
-        
+	v4mState *s = (v4mState *)f->data;
+	ms_average_fps_init(&s->afps, "QuickTime capture average fps = %f");
+	v4m_start(f,NULL);    
 }
 
 static void v4m_postprocess(MSFilter *f) {
+	v4mState *s = (v4mState *)f->data;
 	v4m_stop(f,NULL);
+	ms_average_fps_init(&s->afps, "QuickTime capture average fps = %f");
 }
 
 static int v4m_set_fps(MSFilter *f, void *arg) {
@@ -418,6 +418,12 @@ static int v4m_set_fps(MSFilter *f, void *arg) {
 	return 0;
 }
 
+static int v4m_get_fps(MSFilter *f, void *arg) {
+	v4mState *s = (v4mState *)f->data;
+	*((float *)arg) = ms_average_fps_get(&s->afps);
+	return 0;
+}
+
 static int v4m_get_pix_fmt(MSFilter *f,void *arg) {
 	NSAutoreleasePool* myPool = [[NSAutoreleasePool alloc] init];
 	v4mState *s = (v4mState*)f->data;
@@ -444,6 +450,7 @@ static int v4m_get_vsize(MSFilter *f, void *arg) {
 
 static MSFilterMethod methods[] = {
 	{	MS_FILTER_SET_FPS		,	v4m_set_fps		},
+	{	MS_FILTER_GET_FPS		,	v4m_get_fps		},
 	{	MS_FILTER_GET_PIX_FMT	,	v4m_get_pix_fmt	},
 	{	MS_FILTER_SET_VIDEO_SIZE, 	v4m_set_vsize	},
 	{	MS_V4L_START			,	v4m_start		},
