@@ -17,13 +17,9 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#ifdef HAVE_CONFIG_H
-#include "mediastreamer-config.h"
-#endif
 
 #include <math.h>
-
-#include "mediastreamer2/mediastream.h"
+#include "common.h"
 #include "mediastreamer2/msequalizer.h"
 #include "mediastreamer2/msvolume.h"
 #ifdef VIDEO_ENABLED
@@ -55,12 +51,15 @@ extern void ms_set_video_stream(VideoStream* video);
 #ifdef HAVE_X264
 extern void libmsx264_init();
 #endif
+#ifdef HAVE_OPENH264
+extern void libmsopenh264_init();
+#endif
 #ifdef HAVE_SILK
 extern void libmssilk_init();
 #endif
-#ifdef HAVE_ISAC
-extern void libmsisac_init();
-#endif  
+#ifdef HAVE_WEBRTC
+extern void libmswebrtc_init();
+#endif
 #endif
 
 #ifdef ANDROID
@@ -70,8 +69,13 @@ extern void libmsisac_init();
 
 #include <ortp/b64.h>
 
-#define MEDIASTREAM_MAX_ICE_CANDIDATES 3
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
+
+
+#define MEDIASTREAM_MAX_ICE_CANDIDATES 3
 
 static int cond=1;
 
@@ -95,10 +99,7 @@ typedef struct _MediastreamDatas {
 	bool_t eq;
 	bool_t is_verbose;
 	int device_rotation;
-
-#ifdef VIDEO_ENABLED
 	VideoStream *video;
-#endif
 	char * capture_card;
 	char * playback_card;
 	char * camera;
@@ -108,9 +109,14 @@ typedef struct _MediastreamDatas {
 	bool_t two_windows;
 	bool_t el;
 	bool_t use_rc;
+
 	bool_t enable_srtp;
 	bool_t interactive;
-	bool_t pad[2];
+	bool_t enable_avpf;
+	bool_t enable_rtcp;
+
+	bool_t freeze_on_error;
+	bool_t pad[3];
 	float el_speed;
 	float el_thres;
 	float el_force;
@@ -125,12 +131,11 @@ typedef struct _MediastreamDatas {
 	int ec_len_ms, ec_delay_ms, ec_framesize;
 	char* srtp_local_master_key;
 	char* srtp_remote_master_key;
-	int netsim_bw;
-	int netsim_lossrate;
+	OrtpNetworkSimulatorParams netsim;
 	float zoom;
 	float zoom_cx, zoom_cy;
-	
-	AudioStream *audio;	
+
+	AudioStream *audio;
 	PayloadType *pt;
 	RtpSession *session;
 	OrtpEvQueue *q;
@@ -141,6 +146,8 @@ typedef struct _MediastreamDatas {
 	MediastreamIceCandidate ice_remote_candidates[MEDIASTREAM_MAX_ICE_CANDIDATES];
 	int ice_local_candidates_nb;
 	int ice_remote_candidates_nb;
+	char * video_display_filter;
+	FILE * logfile;
 } MediastreamDatas;
 
 
@@ -163,55 +170,65 @@ static bool_t parse_ice_addr(char* addr, char* type, int type_len, char* ip, int
 static void display_items(void *user_data, uint32_t csrc, rtcp_sdes_type_t t, const char *content, uint8_t content_len);
 static void parse_rtcp(mblk_t *m);
 static void parse_events(RtpSession *session, OrtpEvQueue *q);
-static PayloadType* create_custom_payload_type(const char *type, const char *subtype, const char *rate, const char *channels, int number);
-static PayloadType* parse_custom_payload(const char *name);
 static bool_t parse_window_ids(const char *ids, int* video_id, int* preview_id);
 
-const char *usage="mediastream --local <port> --remote <ip:port> \n"
-								"--payload <payload type number or payload name like 'audio/pmcu/8000'>\n"
-								"[ --fmtp <fmtpline> ]\n"
-								"[ --jitter <miliseconds> ]\n"
-								"[ --width <pixels> ]\n"
-								"[ --height <pixels> ]\n"
+const char *usage="mediastream --local <port>\n"
+								"--remote <ip:port> \n"
+								"[--help (display this help) ]\n"
+								"[--payload <payload type number or payload name like 'audio/pmcu/8000'> ]\n"
+								"[ --agc (enable automatic gain control) ]\n"
 								"[ --bitrate <bits per seconds> ]\n"
+								"[ --camera <camera id as listed at startup> ]\n"
+								"[ --capture-card <name> ]\n"
 								"[ --ec (enable echo canceller) ]\n"
-								"[ --ec-tail <echo canceller tail length in ms> ]\n"
 								"[ --ec-delay <echo canceller delay in ms> ]\n"
 								"[ --ec-framesize <echo canceller framesize in samples> ]\n"
-								"[ --agc (enable automatic gain control) ]\n"
-								"[ --ng (enable noise gate)] \n"
-								"[ --ng-threshold <(float) [0-1]> (noise gate threshold) ]\n"
-								"[ --ng-floorgain <(float) [0-1]> (gain applied to the signal when its energy is below the threshold.) ]\n"
-								"[ --capture-card <name> ]\n"
-								"[ --playback-card <name> ]\n"
-								"[ --infile	<input wav file> specify a wav file to be used for input, instead of soundcard ]\n"
-								"[ --outfile <output wav file> specify a wav file to write audio into, instead of soundcard ]\n"
-								"[ --camera <camera id as listed at startup> ]\n"
+								"[ --ec-tail <echo canceller tail length in ms> ]\n"
 								"[ --el (enable echo limiter) ]\n"
-								"[ --el-speed <(float) [0-1]> (gain changes are smoothed with a coefficent) ]\n"
-								"[ --el-thres <(float) [0-1]> (Threshold above which the system becomes active) ]\n"
 								"[ --el-force <(float) [0-1]> (The proportional coefficient controlling the mic attenuation) ]\n"
+								"[ --el-speed <(float) [0-1]> (gain changes are smoothed with a coefficent) ]\n"
 								"[ --el-sustain <(int)> (Time in milliseconds for which the attenuation is kept unchanged after) ]\n"
+								"[ --el-thres <(float) [0-1]> (Threshold above which the system becomes active) ]\n"
 								"[ --el-transmit-thres <(float) [0-1]> (TO BE DOCUMENTED) ]\n"
-								"[ --rc (enable adaptive rate control) ]\n"
-								"[ --zrtp <secrets file> (enable zrtp) ]\n"
-								"[ --verbose (most verbose messages) ]\n"
-								"[ --video-windows-id <video surface:preview surface>]\n"
-								"[ --srtp <local master_key> <remote master_key> (enable srtp, master key is generated if absent from comand line)\n"
-								"[ --netsim-bandwidth <bandwidth limit in bits/s> (simulates a network download bandwidth limit)\n"
-								"[ --netsim-lossrate <0-100> (simulates a network lost rate)\n"
-								"[ --zoom zoomfactor]\n"
+								"[ --fmtp <fmtpline> ]\n"
+								"[ --freeze-on-error (for video, stop upon decoding error until next valid frame) ]\n"
+								"[ --height <pixels> ]\n"
 								"[ --ice-local-candidate <ip:port:[host|srflx|prflx|relay]> ]\n"
 								"[ --ice-remote-candidate <ip:port:[host|srflx|prflx|relay]> ]\n"
+								"[ --infile <input wav file> specify a wav file to be used for input, instead of soundcard ]\n"
+								"[ --interactive (run in interactive mode) ]\n"
+								"[ --jitter <miliseconds> ]\n"
+								"[ --log <file> ]\n"
 								"[ --mtu <mtu> (specify MTU)]\n"
-								"[ --interactive (run in interactive mode)]\n"
-		;
+								"[ --netsim-bandwidth <bandwidth limit in bits/s> (simulates a network download bandwidth limit) ]\n"
+								"[ --netsim-consecutive-loss-probability <0-1> (to simulate bursts of lost packets) ]\n"
+								"[ --netsim-jitter-burst-density <0-10> (density of gap/burst events, 1.0=one gap/burst per second in average) ]\n"
+								"[ --netsim-jitter-strength <0-100> (strength of the jitter simulation) ]\n"
+								"[ --netsim-latency <latency in ms> (simulates a network latency) ]\n"
+								"[ --netsim-lossrate <0-100> (simulates a network lost rate) ]\n"
+								"[ --netsim-mode inbound|outboud (whether network simulation is applied to incoming (default) or outgoing stream) ]\n"
+								"[ --ng (enable noise gate)] \n"
+								"[ --ng-floorgain <(float) [0-1]> (gain applied to the signal when its energy is below the threshold.) ]\n"
+								"[ --ng-threshold <(float) [0-1]> (noise gate threshold) ]\n"
+								"[ --no-avpf ]\n"
+								"[ --no-rtcp ]\n"
+								"[ --outfile <output wav file> specify a wav file to write audio into, instead of soundcard ]\n"
+								"[ --playback-card <name> ]\n"
+								"[ --rc (enable adaptive rate control) ]\n"
+								"[ --srtp <local master_key> <remote master_key> (enable srtp, master key is generated if absent from comand line) ]\n"
+								"[ --verbose (most verbose messages) ]\n"
+								"[ --video-display-filter <name> ]\n"
+								"[ --video-windows-id <video surface:preview surface >]\n"
+								"[ --width <pixels> ]\n"
+								"[ --zoom zoom factor ]\n"
+								"[ --zrtp <secrets file> (enable zrtp) ]\n"
+								;
 
 #if TARGET_OS_IPHONE
 int g_argc;
 char** g_argv;
 static int _main(int argc, char * argv[]);
- 
+
 static void* apple_main(void* data) {
 	 _main(g_argc,g_argv);
 	 return NULL;
@@ -243,17 +260,19 @@ int main(int argc, char * argv[])
 
 	args = init_default_args();
 
-	if (!parse_args(argc, argv, args))
+	if (!parse_args(argc, argv, args)){
+		printf("%s",usage);
 		return 0;
+	}
 
 	setup_media_streams(args);
 
 	mediastream_run_loop(args);
-	
+
 	clear_mediastreams(args);
 
 	free(args);
-	
+
 	return 0;
 }
 
@@ -298,6 +317,8 @@ MediastreamDatas* init_default_args() {
 	args->custom_pt=NULL;
 	args->video_window_id = -1;
 	args->preview_window_id = -1;
+	args->enable_avpf = TRUE;
+	args->enable_rtcp = TRUE;
 	/* starting values echo canceller */
 	args->ec_len_ms=args->ec_delay_ms=args->ec_framesize=0;
 	args->enable_srtp = FALSE;
@@ -310,11 +331,13 @@ MediastreamDatas* init_default_args() {
 	args->pt = NULL;
 	args->q = NULL;
 	args->profile = NULL;
+	args->logfile = NULL;
 
 	args->ice_session = NULL;
 	memset(args->ice_local_candidates, 0, sizeof(args->ice_local_candidates));
 	memset(args->ice_remote_candidates, 0, sizeof(args->ice_remote_candidates));
 	args->ice_local_candidates_nb = args->ice_remote_candidates_nb = 0;
+	args->video_display_filter=NULL;
 
 	return args;
 }
@@ -323,7 +346,7 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 	int i;
 
 	if (argc<4) {
-		printf("%s",usage);
+		ms_error("Expected at least 3 arguments.\n");
 		return FALSE;
 	}
 
@@ -332,51 +355,58 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 	out->vs.height=MS_VIDEO_SIZE_CIF_H;
 
 	for (i=1;i<argc;i++){
-		if (strcmp(argv[i],"--local")==0){
+		if (strcmp(argv[i],"--help")==0 || strcmp(argv[i],"-h")==0) {
+			return FALSE;
+		}else if (strcmp(argv[i],"--local")==0){
+			char *is_invalid;
 			i++;
-			out->localport=atoi(argv[i]);
+			out->localport = strtol(argv[i],&is_invalid,10);
+			if (*is_invalid!='\0'){
+				ms_error("Failed to parse local port '%s'\n",argv[i]);
+				return 0;
+			}
 		}else if (strcmp(argv[i],"--remote")==0){
 			i++;
 			if (!parse_addr(argv[i],out->ip,sizeof(out->ip),&out->remoteport)) {
-				printf("%s",usage);
+				ms_error("Failed to parse remote address '%s'\n",argv[i]);
 				return FALSE;
 			}
-			printf("Remote addr: ip=%s port=%i\n",out->ip,out->remoteport);
+			ms_message("Remote addr: ip=%s port=%i\n",out->ip,out->remoteport);
 		}else if (strcmp(argv[i],"--ice-local-candidate")==0) {
 			MediastreamIceCandidate *candidate;
 			i++;
 			if (out->ice_local_candidates_nb>=MEDIASTREAM_MAX_ICE_CANDIDATES) {
-				printf("Ignore ICE local candidate \"%s\" (maximum %d candidates allowed)\n",argv[i],MEDIASTREAM_MAX_ICE_CANDIDATES);
+				ms_warning("Ignore ICE local candidate \"%s\" (maximum %d candidates allowed)\n",argv[i],MEDIASTREAM_MAX_ICE_CANDIDATES);
 				continue;
 			}
 			candidate=&out->ice_local_candidates[out->ice_local_candidates_nb];
 			if (!parse_ice_addr(argv[i],candidate->type,sizeof(candidate->type),candidate->ip,sizeof(candidate->ip),&candidate->port)) {
-				printf("%s",usage);
+				ms_error("Failed to parse ICE local candidates '%s'\n", argv[i]);
 				return FALSE;
 			}
 			out->ice_local_candidates_nb++;
-			printf("ICE local candidate: type=%s ip=%s port=%i\n",candidate->type,candidate->ip,candidate->port);
+			ms_message("ICE local candidate: type=%s ip=%s port=%i\n",candidate->type,candidate->ip,candidate->port);
 		}else if (strcmp(argv[i],"--ice-remote-candidate")==0) {
 			MediastreamIceCandidate *candidate;
 			i++;
 			if (out->ice_remote_candidates_nb>=MEDIASTREAM_MAX_ICE_CANDIDATES) {
-				printf("Ignore ICE remote candidate \"%s\" (maximum %d candidates allowed)\n",argv[i],MEDIASTREAM_MAX_ICE_CANDIDATES);
+				ms_warning("Ignore ICE remote candidate \"%s\" (maximum %d candidates allowed)\n",argv[i],MEDIASTREAM_MAX_ICE_CANDIDATES);
 				continue;
 			}
 			candidate=&out->ice_remote_candidates[out->ice_remote_candidates_nb];
 			if (!parse_ice_addr(argv[i],candidate->type,sizeof(candidate->type),candidate->ip,sizeof(candidate->ip),&candidate->port)) {
-				printf("%s",usage);
+				ms_error("Failed to parse ICE remote candidates '%s'\n", argv[i]);
 				return FALSE;
 			}
 			out->ice_remote_candidates_nb++;
-			printf("ICE remote candidate: type=%s ip=%s port=%i\n",candidate->type,candidate->ip,candidate->port);
+			ms_message("ICE remote candidate: type=%s ip=%s port=%i\n",candidate->type,candidate->ip,candidate->port);
 		}else if (strcmp(argv[i],"--payload")==0){
 			i++;
 			if (isdigit(argv[i][0])){
 				out->payload=atoi(argv[i]);
 			}else {
 				out->payload=114;
-				out->custom_pt=parse_custom_payload(argv[i]);
+				out->custom_pt=ms_tools_parse_custom_payload(argv[i]);
 			}
 		}else if (strcmp(argv[i],"--fmtp")==0){
 			i++;
@@ -459,15 +489,15 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 		} else if (strcmp(argv[i], "--video-windows-id")==0) {
 			i++;
 			if (!parse_window_ids(argv[i],&out->video_window_id, &out->preview_window_id)) {
-				printf("%s",usage);
+				ms_error("Failed to parse window ids '%s'\n",argv[i]);
 				return FALSE;
 			}
 		} else if (strcmp(argv[i], "--device-rotation")==0) {
 			i++;
 			out->device_rotation=atoi(argv[i]);
 		} else if (strcmp(argv[i], "--srtp")==0) {
-			if (!ortp_srtp_supported()) {
-				ms_error("ortp srtp support not enabled");
+			if (!ms_srtp_supported()) {
+				ms_error("srtp support not enabled");
 				return FALSE;
 			}
 			out->enable_srtp = TRUE;
@@ -479,70 +509,195 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 			}
 		} else if (strcmp(argv[i],"--netsim-bandwidth")==0){
 			i++;
-			out->netsim_bw=atoi(argv[i]);
-		} else if (strcmp(argv[i],"--netsim-lossrate")==0){
+			if (i<argc){
+				out->netsim.max_bandwidth=atoi(argv[i]);
+				out->netsim.enabled=TRUE;
+			}else{
+				ms_error("Missing argument for --netsim-bandwidth");
+				return FALSE;
+			}
+		}else if (strcmp(argv[i],"--netsim-lossrate")==0){
 			i++;
-			out->netsim_lossrate=atoi(argv[i]);
-			if(out->netsim_lossrate < 0) {
-				ms_warning("%d < 0, wrong value for --lost-rate: set to 0", out->netsim_lossrate);
-				out->netsim_lossrate=0;
+			if (i<argc){
+				out->netsim.loss_rate=atoi(argv[i]);
+				if (out->netsim.loss_rate < 0 || out->netsim.loss_rate>100) {
+					ms_error("Loss rate must be between 0 and 100.");
+					return FALSE;
+				}
+				out->netsim.enabled=TRUE;
+			}else{
+				ms_error("Missing argument for --netsim-lossrate");
+				return FALSE;
 			}
-			if(out->netsim_lossrate > 100) {
-				ms_warning("%d > 100, wrong value for --lost-rate: set to 100", out->netsim_lossrate);
-				out->netsim_lossrate=100;
+		}else if (strcmp(argv[i],"--netsim-consecutive-loss-probability")==0){
+			i++;
+			if (i<argc){
+				sscanf(argv[i],"%f",&out->netsim.consecutive_loss_probability);
+				if (out->netsim.consecutive_loss_probability < 0 || out->netsim.consecutive_loss_probability>1) {
+					ms_error("The consecutive loss probability must be between 0 and 1.");
+					return FALSE;
+				}
+
+				out->netsim.enabled=TRUE;
+			}else{
+				ms_error("Missing argument for --netsim-consecutive-loss-probability");
+				return FALSE;
 			}
-		} else if (strcmp(argv[i],"--zoom")==0){
+		}else if (strcmp(argv[i], "--netsim-latency") == 0) {
+			i++;
+			if (i<argc){
+				out->netsim.latency = atoi(argv[i]);
+				out->netsim.enabled=TRUE;
+			}else{
+				ms_error("Missing argument for --netsim-latency");
+				return FALSE;
+			}
+		}else if (strcmp(argv[i], "--netsim-jitter-burst-density") == 0) {
+			i++;
+			if (i<argc){
+				sscanf(argv[i],"%f",&out->netsim.jitter_burst_density);
+				if (out->netsim.jitter_burst_density<0 || out->netsim.jitter_burst_density>10){
+					ms_error("The jitter burst density must be between 0 and 10");
+					return FALSE;
+				}
+				out->netsim.enabled=TRUE;
+			}else{
+				ms_error("Missing argument for --netsim-jitter-burst-density");
+				return FALSE;
+			}
+		}else if (strcmp(argv[i], "--netsim-jitter-strength") == 0) {
+			i++;
+			if (i<argc){
+				sscanf(argv[i],"%f",&out->netsim.jitter_strength);
+				if (out->netsim.jitter_strength<0 || out->netsim.jitter_strength>100){
+					ms_error("The jitter strength must be between 0 and 100.");
+					return FALSE;
+				}
+				out->netsim.enabled=TRUE;
+			}else{
+				ms_error("Missing argument for --netsim-jitter-strength");
+				return FALSE;
+			}
+		}else if (strcmp(argv[i], "--netsim-mode") == 0) {
+			i++;
+			if (i<argc){
+				if (strcmp(argv[i],"inbound")==0)
+					out->netsim.mode=OrtpNetworkSimulatorInbound;
+				else if (strcmp(argv[i],"outbound")==0){
+					out->netsim.mode=OrtpNetworkSimulatorOutbound;
+				}else{
+					ms_error("Invalid value for --netsim-mode");
+					return FALSE;
+				}
+				out->netsim.enabled=TRUE;
+			}else{
+				ms_error("Missing argument for --netsim-dir");
+				return FALSE;
+			}
+		}else if (strcmp(argv[i],"--zoom")==0){
 			i++;
 			if (sscanf(argv[i], "%f,%f,%f", &out->zoom, &out->zoom_cx, &out->zoom_cy) != 3) {
 				ms_error("Invalid zoom triplet");
 				return FALSE;
 			}
-		}else if (strcmp(argv[i],"--mtu")==0){
+		} else if (strcmp(argv[i],"--mtu")==0){
 			i++;
 			if (sscanf(argv[i], "%i", &out->mtu) != 1) {
 				ms_error("Invalid mtu value");
 				return FALSE;
 			}
-		}else if (strcmp(argv[i],"--interactive")==0){
-			i++;
+		} else if (strcmp(argv[i],"--interactive")==0){
 			out->interactive=TRUE;
-		}
-		else if (strcmp(argv[i],"--help")==0){
-			printf("%s",usage);
+		} else if (strcmp(argv[i], "--no-avpf") == 0) {
+			out->enable_avpf = FALSE;
+		} else if (strcmp(argv[i], "--no-rtcp") == 0) {
+			out->enable_rtcp = FALSE;
+		} else if (strcmp(argv[i],"--help")==0) {
+			return FALSE;
+		} else if (strcmp(argv[i],"--video-display-filter")==0) {
+			i++;
+			out->video_display_filter=argv[i];
+		} else if (strcmp(argv[i], "--log") == 0) {
+			i++;
+			out->logfile = fopen(argv[i], "a+");
+		} else if (strcmp(argv[i], "--freeze-on-error") == 0) {
+			out->freeze_on_error=TRUE;
+		} else {
+			ms_error("Unknown option '%s'\n", argv[i]);
 			return FALSE;
 		}
 	}
+	if (out->netsim.jitter_burst_density>0 && out->netsim.max_bandwidth==0){
+		ms_error("Jitter probability settings requires --netsim-bandwidth to be set.");
+		return FALSE;
+	}
 	return TRUE;
 }
 
 
+#ifdef VIDEO_ENABLED
+static void video_stream_event_cb(void *user_pointer, const MSFilter *f, const unsigned int event_id, const void *args) {
+	MediastreamDatas *md = (MediastreamDatas *)user_pointer;
+	switch (event_id) {
+		case MS_VIDEO_DECODER_DECODING_ERRORS:
+			ms_warning("Decoding error on videostream [%p]", md->video);
+			break;
+		case MS_VIDEO_DECODER_FIRST_IMAGE_DECODED:
+			ms_message("First video frame decoded successfully on videostream [%p]", md->video);
+			break;
+	}
+}
+#endif
+
+static MSSndCard *get_sound_card(MSSndCardManager *manager, const char* card_name) {
+	MSSndCard *play = ms_snd_card_manager_get_card(manager,card_name);
+	if (play == NULL) {
+		const MSList *list = ms_snd_card_manager_get_list(manager);
+		char * cards = ms_strdup("");
+		while (list) {
+			MSSndCard *card = (MSSndCard*)list->data;
+			cards = ms_strcat_printf(cards, "- %s\n", ms_snd_card_get_string_id(card));
+			list = list->next;
+		}
+		ms_fatal("Specified card '%s' but could not find it. Available cards are:\n%s", card_name, cards);
+		ms_free(cards);
+	}
+	return play;
+}
+
 void setup_media_streams(MediastreamDatas* args) {
 	/*create the rtp session */
-	OrtpNetworkSimulatorParams params={0};
-#ifdef VIDEO_ENABLED	
+#ifdef VIDEO_ENABLED
 	MSWebCam *cam=NULL;
 #endif
 	ortp_init();
+	if (args->logfile)
+		ortp_set_log_file(args->logfile);
+
 	if (args->is_verbose) {
 		ortp_set_log_level_mask(ORTP_DEBUG|ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	} else {
 		ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	}
 
+	ms_init();
 
 #if TARGET_OS_IPHONE || defined(ANDROID)
 #if defined (HAVE_X264) && defined (VIDEO_ENABLED)
 	libmsx264_init(); /*no plugin on IOS/Android */
 #endif
+#if defined (HAVE_OPENH264) && defined (VIDEO_ENABLED)
+	libmsopenh264_init(); /*no plugin on IOS/Android */
+#endif
 #if defined (HAVE_SILK)
 	libmssilk_init(); /*no plugin on IOS/Android */
 #endif
-#if defined (HAVE_ISAC)
-	libmsisac_init();
-#endif	
+#if defined (HAVE_WEBRTC)
+	libmswebrtc_init();
+#endif
 
 #endif /* IPHONE | ANDROID */
-	
+
 	rtp_profile_set_payload(&av_profile,110,&payload_type_speex_nb);
 	rtp_profile_set_payload(&av_profile,111,&payload_type_speex_wb);
 	rtp_profile_set_payload(&av_profile,112,&payload_type_ilbc);
@@ -550,9 +705,10 @@ void setup_media_streams(MediastreamDatas* args) {
 	rtp_profile_set_payload(&av_profile,114,args->custom_pt);
 	rtp_profile_set_payload(&av_profile,115,&payload_type_lpc1015);
 #ifdef VIDEO_ENABLED
-#if TARGET_OS_IPHONE && defined (HAVE_X264)
-	libmsx264_init(); /*no plugin on IOS*/
-#endif
+	cam=ms_web_cam_new(ms_mire_webcam_desc_get());
+	if (cam) ms_web_cam_manager_add_cam(ms_web_cam_manager_get(), cam);
+	cam=NULL;
+
 	rtp_profile_set_payload(&av_profile,26,&payload_type_jpeg);
 	rtp_profile_set_payload(&av_profile,98,&payload_type_h263_1998);
 	rtp_profile_set_payload(&av_profile,97,&payload_type_theora);
@@ -560,13 +716,12 @@ void setup_media_streams(MediastreamDatas* args) {
 	rtp_profile_set_payload(&av_profile,100,&payload_type_x_snow);
 	rtp_profile_set_payload(&av_profile,102,&payload_type_h264);
 	rtp_profile_set_payload(&av_profile,103,&payload_type_vp8);
-	
+
 	args->video=NULL;
 #endif
 	args->profile=rtp_profile_clone_full(&av_profile);
 	args->q=ortp_ev_queue_new();
 
-	ms_init();
 	if (args->mtu) ms_set_mtu(args->mtu);
 	ms_filter_enable_statistics(TRUE);
 	ms_filter_reset_statistics();
@@ -579,14 +734,23 @@ void setup_media_streams(MediastreamDatas* args) {
 	signal(SIGINT,stop_handler);
 	args->pt=rtp_profile_get_payload(args->profile,args->payload);
 	if (args->pt==NULL){
-		printf("Error: no payload defined with number %i.\n",args->payload);
+		ms_error("No payload defined with number %i.\n",args->payload);
 		exit(-1);
 	}
+	if (args->enable_avpf == TRUE) {
+		PayloadTypeAvpfParams avpf_params;
+		payload_type_set_flag(args->pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
+		avpf_params.features = PAYLOAD_TYPE_AVPF_FIR | PAYLOAD_TYPE_AVPF_PLI | PAYLOAD_TYPE_AVPF_SLI | PAYLOAD_TYPE_AVPF_RPSI;
+		avpf_params.trr_interval = 3000;
+		payload_type_set_avpf_params(args->pt, avpf_params);
+	} else {
+		payload_type_unset_flag(args->pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
+	}
 	if (args->fmtp!=NULL) payload_type_set_send_fmtp(args->pt,args->fmtp);
 	if (args->bitrate>0) args->pt->normal_bitrate=args->bitrate;
-	
+
 	if (args->pt->normal_bitrate==0){
-		printf("Error: no default bitrate specified for codec %s/%i. "
+		ms_error("Default bitrate specified for codec %s/%i. "
 			"Please specify a network bitrate with --bitrate option.\n",args->pt->mime_type,args->pt->clock_rate);
 		exit(-1);
 	}
@@ -596,44 +760,44 @@ void setup_media_streams(MediastreamDatas* args) {
 		// default profile require key-length = 30 bytes
 		//  -> input : 40 b64 encoded bytes
 		if (!args->srtp_local_master_key) {
-			uint8_t tmp[30];			
-			ortp_crypto_get_random(tmp, 30);
+			char tmp[30];
+			snprintf(tmp,sizeof(tmp),"%08x%08x%08x%08x",rand(),rand(),rand(),rand());
 			args->srtp_local_master_key = (char*) malloc(41);
 			b64_encode((const char*)tmp, 30, args->srtp_local_master_key, 40);
 			args->srtp_local_master_key[40] = '\0';
 			ms_message("Generated local srtp key: '%s'", args->srtp_local_master_key);
 		}
 		if (!args->srtp_remote_master_key) {
-			uint8_t tmp[30];			
-			ortp_crypto_get_random(tmp, 30);
+			char tmp[30];
+			snprintf(tmp,sizeof(tmp),"%08x%08x%08x%08x",rand(),rand(),rand(),rand());
 			args->srtp_remote_master_key = (char*) malloc(41);
 			b64_encode((const char*)tmp, 30, args->srtp_remote_master_key, 40);
 			args->srtp_remote_master_key[40] = '\0';
 			ms_message("Generated remote srtp key: '%s'", args->srtp_remote_master_key);
 		}
-	}	
+	}
 
 	if (args->pt->type!=PAYLOAD_VIDEO){
 		MSSndCardManager *manager=ms_snd_card_manager_get();
 		MSSndCard *capt= args->capture_card==NULL ? ms_snd_card_manager_get_default_capture_card(manager) :
-				ms_snd_card_manager_get_card(manager,args->capture_card);
-		MSSndCard *play= args->playback_card==NULL ? ms_snd_card_manager_get_default_playback_card(manager) :
-				ms_snd_card_manager_get_card(manager,args->playback_card);
+				get_sound_card(manager,args->capture_card);
+		MSSndCard *play= args->playback_card==NULL ? ms_snd_card_manager_get_default_capture_card(manager) :
+				get_sound_card(manager,args->playback_card);
 		args->audio=audio_stream_new(args->localport,args->localport+1,ms_is_ipv6(args->ip));
 		audio_stream_enable_automatic_gain_control(args->audio,args->agc);
 		audio_stream_enable_noise_gate(args->audio,args->use_ng);
 		audio_stream_set_echo_canceller_params(args->audio,args->ec_len_ms,args->ec_delay_ms,args->ec_framesize);
 		audio_stream_enable_echo_limiter(args->audio,args->el);
 		audio_stream_enable_adaptive_bitrate_control(args->audio,args->use_rc);
-		printf("Starting audio stream.\n");
+		ms_message("Starting audio stream.\n");
 
-		audio_stream_start_full(args->audio,args->profile,args->ip,args->remoteport,args->ip,args->remoteport+1, args->payload, args->jitter,args->infile,args->outfile,
-		                        args->outfile==NULL ? play : NULL ,args->infile==NULL ? capt : NULL,args->infile!=NULL ? FALSE: args->ec);
+		audio_stream_start_full(args->audio,args->profile,args->ip,args->remoteport,args->ip,args->enable_rtcp?args->remoteport+1:-1, args->payload, args->jitter,args->infile,args->outfile,
+								args->outfile==NULL ? play : NULL ,args->infile==NULL ? capt : NULL,args->infile!=NULL ? FALSE: args->ec);
 
 		if (args->ice_local_candidates_nb || args->ice_remote_candidates_nb) {
 			args->audio->ms.ice_check_list = ice_check_list_new();
-			rtp_session_set_pktinfo(args->audio->ms.session,TRUE);
-			ice_session_add_check_list(args->ice_session, args->audio->ms.ice_check_list);
+			rtp_session_set_pktinfo(args->audio->ms.sessions.rtp_session,TRUE);
+			ice_session_add_check_list(args->ice_session, args->audio->ms.ice_check_list, 0);
 		}
 		if (args->ice_local_candidates_nb) {
 			MediastreamIceCandidate *candidate;
@@ -682,23 +846,23 @@ void setup_media_streams(MediastreamDatas* args) {
 				}
 			}
 
-            #ifndef TARGET_OS_IPHONE
+			#ifndef TARGET_OS_IPHONE
 			if (args->zrtp_secrets != NULL) {
-				OrtpZrtpParams params;
+				MSZrtpParams params;
 				params.zid_file=args->zrtp_secrets;
 				audio_stream_enable_zrtp(args->audio,&params);
 			}
-            #endif
+			#endif
 
-			args->session=args->audio->ms.session;
+			args->session=args->audio->ms.sessions.rtp_session;
 		}
-		
+
 		if (args->enable_srtp) {
-			ms_message("SRTP enabled: %d", 
+			ms_message("SRTP enabled: %d",
 				audio_stream_enable_srtp(
-					args->audio, 
-					AES_128_SHA1_80,
-					args->srtp_local_master_key, 
+					args->audio,
+					MS_AES_128_SHA1_80,
+					args->srtp_local_master_key,
 					args->srtp_remote_master_key));
 		}
 	}else{
@@ -713,6 +877,9 @@ void setup_media_streams(MediastreamDatas* args) {
 		}
 		ms_message("Starting video stream.\n");
 		args->video=video_stream_new(args->localport, args->localport+1, ms_is_ipv6(args->ip));
+		if (args->video_display_filter)
+			video_stream_set_display_filter_name(args->video, args->video_display_filter);
+
 #ifdef ANDROID
 		if (args->device_rotation >= 0)
 			video_stream_set_device_rotation(args->video, args->device_rotation);
@@ -726,7 +893,8 @@ void setup_media_streams(MediastreamDatas* args) {
 		NSUInteger cpucount = [[NSProcessInfo processInfo] processorCount];
 		ms_set_cpu_count(cpucount);
 #endif
-
+		video_stream_set_event_callback(args->video,video_stream_event_cb, args);
+		video_stream_set_freeze_on_error(args->video,args->freeze_on_error);
 		video_stream_enable_adaptive_bitrate_control(args->video,args->use_rc);
 		if (args->camera)
 			cam=ms_web_cam_manager_get_cam(ms_web_cam_manager_get(),args->camera);
@@ -734,23 +902,23 @@ void setup_media_streams(MediastreamDatas* args) {
 			cam=ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get());
 		video_stream_start(args->video,args->profile,
 					args->ip,args->remoteport,
-					args->ip,args->remoteport+1,
+					args->ip,args->enable_rtcp?args->remoteport+1:-1,
 					args->payload,
 					args->jitter,cam
 					);
-		args->session=args->video->ms.session;
+		args->session=args->video->ms.sessions.rtp_session;
 
 		ms_filter_call_method(args->video->output,MS_VIDEO_DISPLAY_ZOOM, zoom);
 		if (args->enable_srtp) {
-			ms_message("SRTP enabled: %d", 
+			ms_message("SRTP enabled: %d",
 				video_stream_enable_strp(
-					args->video, 
-					AES_128_SHA1_80,
-					args->srtp_local_master_key, 
+					args->video,
+					MS_AES_128_SHA1_80,
+					args->srtp_local_master_key,
 					args->srtp_remote_master_key));
 		}
 #else
-		printf("Error: video support not compiled.\n");
+		ms_error("Error: video support not compiled.\n");
 #endif
 	}
 	ice_session_set_base_for_srflx_candidates(args->ice_session);
@@ -759,18 +927,9 @@ void setup_media_streams(MediastreamDatas* args) {
 	ice_session_choose_default_remote_candidates(args->ice_session);
 	ice_session_start_connectivity_checks(args->ice_session);
 
-	if (args->netsim_bw>0){
-		params.enabled=TRUE;
-		params.max_bandwidth=args->netsim_bw;
+	if (args->netsim.enabled){
+		rtp_session_enable_network_simulation(args->session,&args->netsim);
 	}
-	if (args->netsim_lossrate>0){
-		params.enabled=TRUE;
-		params.loss_rate=args->netsim_lossrate;
-	}
-	if (params.enabled){
-		rtp_session_enable_network_simulation(args->session,&params);
-	}
-
 }
 
 
@@ -778,12 +937,12 @@ static void mediastream_tool_iterate(MediastreamDatas* args) {
 #ifndef WIN32
 	struct pollfd pfd;
 	int err;
-	
+
 	if (args->interactive){
 		pfd.fd=STDIN_FILENO;
 		pfd.events=POLLIN;
 		pfd.revents=0;
-	
+
 		err=poll(&pfd,1,10);
 		if (err==1 && (pfd.revents & POLLIN)){
 			char commands[128];
@@ -791,7 +950,7 @@ static void mediastream_tool_iterate(MediastreamDatas* args) {
 			commands[127]='\0';
 			ms_sleep(1);  /* ensure following text be printed after ortp messages */
 			if (args->eq)
-			printf("\nPlease enter equalizer requests, such as 'eq active 1', 'eq active 0', 'eq 1200 0.1 200'\n");
+			ms_message("\nPlease enter equalizer requests, such as 'eq active 1', 'eq active 0', 'eq 1200 0.1 200'\n");
 
 			if (fgets(commands,sizeof(commands)-1,stdin)!=NULL){
 				int active,freq,freq_width;
@@ -799,13 +958,13 @@ static void mediastream_tool_iterate(MediastreamDatas* args) {
 				float gain;
 				if (sscanf(commands,"eq active %i",&active)==1){
 					audio_stream_enable_equalizer(args->audio,active);
-					printf("OK\n");
+					ms_message("OK\n");
 				}else if (sscanf(commands,"eq %i %f %i",&freq,&gain,&freq_width)==3){
 					audio_stream_equalizer_set_gain(args->audio,freq,gain,freq_width);
-					printf("OK\n");
+					ms_message("OK\n");
 				}else if (sscanf(commands,"eq %i %f",&freq,&gain)==2){
 					audio_stream_equalizer_set_gain(args->audio,freq,gain,0);
-					printf("OK\n");
+					ms_message("OK\n");
 				}else if (strstr(commands,"dump")){
 					int n=0,i;
 					float *t;
@@ -814,19 +973,21 @@ static void mediastream_tool_iterate(MediastreamDatas* args) {
 					ms_filter_call_method(args->audio->equalizer,MS_EQUALIZER_DUMP_STATE,t);
 					for(i=0;i<n;++i){
 						if (fabs(t[i]-1)>0.01){
-						printf("%i:%f:0 ",(i*args->pt->clock_rate)/(2*n),t[i]);
+						ms_message("%i:%f:0 ",(i*args->pt->clock_rate)/(2*n),t[i]);
 						}
 					}
-					printf("\nOK\n");
+					ms_message("\nOK\n");
 				}else if (sscanf(commands,"lossrate %i",&intarg)==1){
-					OrtpNetworkSimulatorParams params={0};
-					params.enabled=TRUE;
-					params.loss_rate=intarg;
-					rtp_session_enable_network_simulation(args->session,&params);
-				}
-				else if (strstr(commands,"quit")){
+					args->netsim.enabled=TRUE;
+					args->netsim.loss_rate=intarg;
+					rtp_session_enable_network_simulation(args->session,&args->netsim);
+				}else if (sscanf(commands,"bandwidth %i",&intarg)==1){
+					args->netsim.enabled=TRUE;
+					args->netsim.max_bandwidth=intarg;
+					rtp_session_enable_network_simulation(args->session,&args->netsim);
+				}else if (strstr(commands,"quit")){
 					cond=0;
-				}else printf("Cannot understand this.\n");
+				}else ms_warning("Cannot understand this.\n");
 			}
 		}else if (err==-1 && errno!=EINTR){
 			ms_fatal("mediastream's poll() returned %s",strerror(errno));
@@ -855,7 +1016,7 @@ void mediastream_run_loop(MediastreamDatas* args) {
 	while(cond)
 	{
 		int n;
-		for(n=0;n<100 && cond;++n){
+		for(n=0;n<500 && cond;++n){
 			mediastream_tool_iterate(args);
 #if defined(VIDEO_ENABLED)
 			if (args->video) video_stream_iterate(args->video);
@@ -864,11 +1025,24 @@ void mediastream_run_loop(MediastreamDatas* args) {
 		}
 		rtp_stats_display(rtp_session_get_stats(args->session),"RTP stats");
 		if (args->session){
+			float audio_load = 0;
+			float video_load = 0;
+
 			ms_message("Bandwidth usage: download=%f kbits/sec, upload=%f kbits/sec\n",
-				rtp_session_compute_recv_bandwidth(args->session)*1e-3,
-				rtp_session_compute_send_bandwidth(args->session)*1e-3);
+				rtp_session_get_recv_bandwidth(args->session)*1e-3,
+				rtp_session_get_send_bandwidth(args->session)*1e-3);
+
+			if (args->audio) {
+				audio_load = ms_ticker_get_average_load(args->audio->ms.sessions.ticker);
+			}
+#if defined(VIDEO_ENABLED)
+			if (args->video) {
+				video_load = ms_ticker_get_average_load(args->video->ms.sessions.ticker);
+			}
+#endif
+			ms_message("Thread processing load: audio=%f\tvideo=%f", audio_load, video_load);
 			parse_events(args->session,args->q);
-			ms_message("Quality indicator : %f\n",args->audio ? audio_stream_get_quality_rating(args->audio) : -1);
+			ms_message("Quality indicator : %f\n",args->audio ? audio_stream_get_quality_rating(args->audio) : media_stream_get_quality_rating((MediaStream*)args->video));
 		}
 	}
 }
@@ -891,6 +1065,9 @@ void clear_mediastreams(MediastreamDatas* args) {
 	ortp_ev_queue_destroy(args->q);
 	rtp_profile_destroy(args->profile);
 
+	if (args->logfile)
+		fclose(args->logfile);
+
 	ms_exit();
 }
 
@@ -905,8 +1082,8 @@ JNIEXPORT jint JNICALL  JNI_OnLoad(JavaVM *ajvm, void *reserved)
 
 JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_setVideoWindowId
   (JNIEnv *env, jobject obj, jobject id, jint _args) {
-	MediastreamDatas* args =  (MediastreamDatas*)_args;
 #ifdef VIDEO_ENABLED
+	MediastreamDatas* args =  (MediastreamDatas*)_args;
 	if (!args->video)
 		return;
 	video_stream_set_native_window_id(args->video,(unsigned long)id);
@@ -915,8 +1092,8 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_setVi
 
 JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_setVideoPreviewWindowId
   (JNIEnv *env, jobject obj, jobject id, jint _args) {
-	MediastreamDatas* args =  (MediastreamDatas*)_args;
 #ifdef VIDEO_ENABLED
+	MediastreamDatas* args =  (MediastreamDatas*)_args;
 	if (!args->video)
 		return;
 	video_stream_set_native_preview_window_id(args->video,(unsigned long)id);
@@ -925,8 +1102,8 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_setVi
 
 JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_setDeviceRotation
   (JNIEnv *env, jobject thiz, jint rotation, jint _args) {
-	MediastreamDatas* args =  (MediastreamDatas*)_args;
 #ifdef VIDEO_ENABLED
+	MediastreamDatas* args =  (MediastreamDatas*)_args;
 	if (!args->video)
 		return;
 	video_stream_set_device_rotation(args->video, rotation);
@@ -935,8 +1112,8 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_setDe
 
 JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_changeCamera
   (JNIEnv *env, jobject obj, jint camId, jint _args) {
-	MediastreamDatas* args =  (MediastreamDatas*)_args;
 #ifdef VIDEO_ENABLED
+	MediastreamDatas* args =  (MediastreamDatas*)_args;
 	if (!args->video)
 		return;
 	char* id = (char*)malloc(15);
@@ -1020,14 +1197,24 @@ static bool_t parse_addr(const char *addr, char *ip, int len, int *port)
 	const char *p;
 
 	*port=0;
-	semicolon=strchr(addr,':');
+
 	for (p=addr+strlen(addr)-1;p>addr;p--){
 		if (*p==':') {
 			semicolon=p;
 			break;
 		}
 	}
-	if (semicolon==NULL) return FALSE;
+	/*if no semicolon is present, we can assume that user provided only port*/
+	if (semicolon==NULL) {
+		const char *localhost = "127.0.0.1";
+		char * end;
+		*port = strtol(addr, &end, 10);
+		if (*end != '\0' || end == addr) {
+			return FALSE;
+		}
+		strncpy(ip,localhost, MIN(len, strlen(localhost)));
+		return TRUE;
+	}
 	iplen=semicolon-addr;
 	slen=MIN(iplen,len-1);
 	strncpy(ip,addr,slen);
@@ -1104,61 +1291,6 @@ static void parse_events(RtpSession *session, OrtpEvQueue *q){
 	}
 }
 
-static PayloadType* create_custom_payload_type(const char *type, const char *subtype, const char *rate, const char *channels, int number){
-	PayloadType *pt=payload_type_new();
-	if (strcasecmp(type,"audio")==0){
-		pt->type=PAYLOAD_AUDIO_PACKETIZED;
-	}else if (strcasecmp(type,"video")==0){
-		pt->type=PAYLOAD_VIDEO;
-	}else{
-		fprintf(stderr,"Unsupported payload type should be audio or video, not %s\n",type);
-		exit(-1);
-	}
-	pt->mime_type=ms_strdup(subtype);
-	pt->clock_rate=atoi(rate);
-	pt->channels=atoi(channels);
-	return pt;	
-}
-
-static PayloadType* parse_custom_payload(const char *name){
-	char type[64]={0};
-	char subtype[64]={0};
-	char clockrate[64]={0};
-	char nchannels[64];
-	char *separator;
-
-	if (strlen(name)>=sizeof(clockrate)-1){
-		fprintf(stderr,"Cannot parse %s: too long.\n",name);
-		exit(-1);
-	}
-
-	separator=strchr(name,'/');
-	if (separator){
-		char *separator2;
-
-		strncpy(type,name,separator-name);
-		separator2=strchr(separator+1,'/');
-		if (separator2){
-			char *separator3;
-
-			strncpy(subtype,separator+1,separator2-separator-1);
-			separator3=strchr(separator2+1,'/');
-			if (separator3){
-				strncpy(clockrate,separator2+1,separator3-separator2-1);
-				strcpy(nchannels,separator3+1);
-			} else {
-				nchannels[0]='1';
-				nchannels[1]='\0';
-				strcpy(clockrate,separator2+1);
-			}
-			fprintf(stdout,"Found custom payload type=%s, mime=%s, clockrate=%s nchannels=%s\n", type, subtype, clockrate, nchannels);
-			return create_custom_payload_type(type,subtype,clockrate,nchannels,114);
-		}
-	}
-	fprintf(stderr,"Error parsing payload name %s.\n",name);
-	exit(-1);
-}
-
 static bool_t parse_window_ids(const char *ids, int* video_id, int* preview_id)
 {
 	char* copy = strdup(ids);
